#include "MySQLDS.h"
#include <stdio.h>
#include <stack>
#include <vector>
#include <list>
#include <ctype.h>
#include <utility>
#if MACHINE_LINUX
#include <memory>
#endif

#if MACHINE_LINUX
#define LINUXFUBAR // linuxfubar::
#else
#define LINUXFUBAR
#endif

const char * kRowCountVar = "__updated_count__";

#ifdef WIN32
#define strcasecmp stricmp
#endif

#define DEBUG_SQL

MYSQL * local_doConnect( lasso_request_t token );

void local_addReturnFields( lasso_request_t token, string & to );
void local_addSearchCriteria( lasso_request_t token, string & to, LP_TypeDesc defaultOp, bool isFindAll );
void local_addUpdateCriteria( lasso_request_t token, string & to );
void local_addSortOrder( lasso_request_t token, string & to );
void local_addSearchValue( string & to, const char * value, unsigned int size, LP_TypeDesc op );
bool local_addPrimaryKeySearch( lasso_request_t token, string & to );
void local_addColumnInfo( lasso_request_t token, MYSQL_RES * res, MYSQL * mysql, const char * db, const char * table );
void local_addOperator( string & to, LP_TypeDesc op, const char * value, unsigned int size );
void local_addInsertCriteria( lasso_request_t token, string & query );

void registerLassoModule()
{
	lasso_registerDSModule( "MySQLDS", local_mysqlds_func, 0 );
	lasso_log(LOG_LEVEL_CRITICAL, "MySQLConnector using MySQL client: %s", LINUXFUBAR mysql_get_client_info());
}

inline osError CheckError(lasso_request_t token, MYSQL * m)
{
	if (LINUXFUBAR mysql_errno(m))
	{
		lasso_log(LOG_LEVEL_WARNING, "MySQLDS: error from mysql: %d, %s", LINUXFUBAR mysql_errno(m), LINUXFUBAR mysql_error(m));
		lasso_setResultMessage(token, LINUXFUBAR mysql_error(m));
		return LINUXFUBAR mysql_errno(m);
	}
	return osErrNoErr;
}

// returns immediately if stat[*ioPos] == scanChar
inline void SQLMovePastChar( const char * stat, int * ioPos, int maxLen, char scanChar )
{
	while( *ioPos < maxLen )
	{
		if ( stat[*ioPos] == scanChar )
		{
			++*ioPos;
			break;
		}
		++*ioPos;
	}
}


inline bool FindStat( const char * stat, int * start, int * finish, int maxLen )
{
	if ( *start == -1 )
		*start = 0, *finish = 0;
	else
		*start = ++*finish;
	if ( *start >= maxLen )
		return false;
	
	while ( *finish < maxLen )
	{
		if ( stat[*finish] == '\'' )
		{
			++*finish;
			SQLMovePastChar(stat, finish, maxLen, '\'');
		}
		else if ( stat[*finish] == '"' )
		{
			++*finish;
			SQLMovePastChar(stat, finish, maxLen, '"');
		}
		else if ( stat[*finish] == ';' )
		{
			break;
		}
		else
			++*finish;
	}
	while( *start != *finish && isspace(stat[*start]) )
	{
		++*start;
	}
	if ( *start == *finish )
	{
		return FindStat(stat, start, finish, maxLen);
	}
	return true;
}

MYSQL * local_doConnect( lasso_request_t token )
{
	// updated to work with LP7 datasource connection caching
	lasso_dsconnection_t test = NULL;
	if (lasso_getDSConnection(token, &test)==osErrNoErr && test != NULL)
	{		
		return reinterpret_cast<MYSQL*>(test);
	}
	auto_lasso_value_t host, auth;
	lasso_getDataHost(token, &host, &auth);

	MYSQL * worked = NULL;
	
	worked = LINUXFUBAR mysql_init(NULL);

	if ( LINUXFUBAR mysql_real_connect(worked, host.name, auth.name, auth.data, NULL, atoi(host.data), "/tmp/mysql.sock", 0) == NULL )
	{
		lasso_log(LOG_LEVEL_CRITICAL, "MySQLDS failed connecting to MySQL. errno=%d \"%s\"", LINUXFUBAR mysql_errno(worked), LINUXFUBAR mysql_error(worked));
		LINUXFUBAR mysql_close(worked);
		worked = NULL;
	}
	else
	{
//	  lasso_log(LOG_LEVEL_DETAIL, "MYSQLDS: connecting %s %s...", LINUXFUBAR mysql_get_host_info(worked), LINUXFUBAR mysql_get_server_info(worked));
//		lasso_log(LOG_LEVEL_DETAIL, "MYSQLDS: done connecting %s...", host.name);
		lasso_setDSConnection(token, reinterpret_cast<lasso_dsconnection_t>(worked));
	}
	return worked;
}

osError local_mysqlds_func( lasso_request_t token, datasource_action_t action, const auto_lasso_value_t * param )
{
	osError err = osErrNoErr;
	switch( action )
	{
		case datasourceInit:
			// nothing!
			break;
		case datasourceTerm:
			// nothing!
			break;
		case datasourceCloseConnection: // connections only get closed through here
			{
				lasso_dsconnection_t conn = NULL;
				if ( lasso_getDSConnection(token, &conn) == osErrNoErr && conn )
				{
					LINUXFUBAR mysql_close(reinterpret_cast<MYSQL*>(conn));
				}
			}
			break;
		case datasourceTickle:
			{
				MYSQL * conn = local_doConnect(token);
				auto_lasso_value_t db;

				lasso_getDataSourceName(token, &db, NULL, NULL);
				LINUXFUBAR mysql_select_db(conn, db.name);
			}
			break;
		case datasourceNames:
			{
				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;
				
				MYSQL_RES * res = NULL;
				MYSQL_ROW row;
				
				if ( LINUXFUBAR mysql_query(mysql, "show databases") == 0 )
				{
					res = LINUXFUBAR mysql_store_result(mysql);
					if ( res != NULL )
					{
						while( (row = LINUXFUBAR mysql_fetch_row(res)) != NULL )
						{
							lasso_addDataSourceResult( token, row[0] );
						}
						LINUXFUBAR mysql_free_result(res);
					}
				}
//				mysql_close(mysql);				
			}
			break;
		case datasourceTableNames:
			{
				MYSQL_RES * res = NULL;

				auto_lasso_value_t db;

				lasso_getDataSourceName(token, &db, NULL, NULL);
				
				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				if ( LINUXFUBAR mysql_select_db(mysql, db.data) == 0 )
				{
					res = LINUXFUBAR mysql_list_tables(mysql, NULL);
					if ( res != NULL )
					{
						MYSQL_ROW row;
						while( (row = LINUXFUBAR mysql_fetch_row(res)) != NULL )
						{
							lasso_addDataSourceResult(token, row[0]);
						}
						LINUXFUBAR mysql_free_result(res);
					}
				}
				err = CheckError(token, mysql);
			}
			break;
		case datasourceSearch:
		case datasourceFindAll:
			{
				auto_lasso_value_t dbName, tableName, test;
				
				string query;
			
				lasso_getDataSourceName( token, &dbName, NULL, NULL );
				lasso_getTableName( token, &tableName );
					
				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				/* SELECT (<result_fields>) FROM <databasename>.<table_name> WHERE <search_fields> ORDER BY <sort_fields> */
				query.append("SELECT ");
				
				if ( lasso_findInputColumn( token, "-distinct", &test ) == osErrNoErr )
				{
					query.append("DISTINCT ");
				}

				local_addReturnFields( token, query );
				
				query.append(" FROM ").append( dbName.data ).append(".").append( tableName.data );

				int tmp;
				lasso_getRowID(token, &tmp);
				if ( tmp != kLassoAny )
				{
					local_addSearchCriteria( token, query, lpTypeNull, action == datasourceFindAll );
	#ifdef DEBUG_SQL
					lasso_log(LOG_LEVEL_DETAIL, "MySQLDS made query: %s", query.c_str());
	#endif				
					if ( LINUXFUBAR mysql_query(mysql, query.c_str()) == 0 )
					{
						MYSQL_ROW row;
						MYSQL_RES * res = LINUXFUBAR mysql_store_result(mysql);
							
						if ( res != NULL )
						{
							int num = LINUXFUBAR mysql_num_fields(res), skip=0, max=0, found;
							// we have to set the column info
							local_addColumnInfo( token, res, mysql, dbName.data, tableName.data );
					
							// if the query used LIMIT then don't do skip and max. just get them all
							if ( lasso_findInputColumn( token, "-uselimit", &test ) == osErrNoErr )
							{
								max = -1;
							}
							else
							{
								lasso_getSkipRows( token, &skip );
								lasso_getMaxRows( token, &max );
							}

							found = (int)LINUXFUBAR mysql_num_rows(res);

							lasso_setNumRowsFound( token, found );

							if ( max > -1 && skip > 0 )
								LINUXFUBAR mysql_data_seek( res, skip );

							while( (row = LINUXFUBAR mysql_fetch_row(res)) != NULL && (max == -1 || max--) )
							{
								unsigned long * lengths = LINUXFUBAR mysql_fetch_lengths(res);
								lasso_addResultRow( token, (const char**)row, (unsigned int*)lengths, num );
							}
							LINUXFUBAR mysql_free_result(res);
						}
					}
				}
				else
				{
					query.append(" ORDER BY RAND() LIMIT 1");
	#ifdef DEBUG_SQL
					lasso_log(LOG_LEVEL_DETAIL, "MySQLDS made query: %s", query.c_str());
	#endif
					MYSQL_ROW row;
					MYSQL_RES * res = NULL;
			
					int result = LINUXFUBAR mysql_query(mysql, query.c_str());
					if ( result == 0 )
					{
						res = LINUXFUBAR mysql_store_result(mysql);
					}	 

					if ( res != NULL )
					{
						int num = LINUXFUBAR mysql_num_fields(res);
						// we have to set the column info
						local_addColumnInfo( token, res, mysql, dbName.data, tableName.data );
						int found = (int)LINUXFUBAR mysql_num_rows(res);

						lasso_setNumRowsFound( token, found );

						row = LINUXFUBAR mysql_fetch_row(res);
						if ( row != NULL )
						{
							unsigned long * lengths = LINUXFUBAR mysql_fetch_lengths(res);
							lasso_addResultRow( token, (const char**)row, (unsigned int*)lengths, num );
						}
						LINUXFUBAR mysql_free_result(res);
					}					
				}				
				err = CheckError(token, mysql);
			}
			break;
		case datasourceAdd:
			 {				 
				auto_lasso_value_t dbName, tableName, val;

				lasso_getDataSourceName( token, &dbName, NULL, NULL );
				lasso_getTableName( token, &tableName );
				
				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				string query;

				query.append("INSERT INTO ").append(dbName.data).append(".").append(tableName.data);

				int num, i;
				
				lasso_getInputColumnCount( token, &num );

				std::vector<auto_lasso_value_t> validValues;

				for ( i = 0; i < num; ++i )
				{
					if ( lasso_getInputColumn( token, i, &val ) == osErrNoErr )
					{
						if ( val.data != NULL && strlen(val.data) > 0 )
							validValues.push_back(val);
					}
					else
						break;
				}

				num = validValues.size();
				if ( num > 0 )
				{
					query.append(" (");
					for ( i = 0; i < num; ++i )
					{
						query.append(validValues[i].name);
						if ( i + 1 < num )
							query.append(",");
					}
					query.append(") VALUES (");

					for ( i = 0; i < num; ++i )
					{
						local_addSearchValue( query, validValues[i].data, validValues[i].dataSize, lpTypeNull);
						
						if ( i + 1 < num )
							query.append(",");
					}
					query.append(")");
				}
#ifdef DEBUG_SQL
				lasso_log(LOG_LEVEL_DETAIL, "MySQLDS made query: %s", query.c_str());
#endif
				if (LINUXFUBAR mysql_query( mysql, query.c_str() ) == 0)
				{
					if ( LINUXFUBAR mysql_affected_rows(mysql) == 0 )
					{
						err = osErrWebAddError;
						lasso_setVariable( token, kRowCountVar, "0", 0 );
					}
					else
					{
						char rowcount[16] = "";      
						sprintf( rowcount, "%lu", (unsigned long)LINUXFUBAR mysql_affected_rows(mysql) );
						lasso_setVariable( token, kRowCountVar, rowcount, 0 );
					}
					if( err == osErrNoErr )
					{
						lasso_setRowID( token, (unsigned long) LINUXFUBAR mysql_insert_id( mysql ) );
						int c;
						auto_lasso_value_t key;
						lasso_getMaxRows(token, &c);
						if ( c > 0 && lasso_getPrimaryKeyColumn( token, &key ) == osErrNoErr && key.nameSize > 0 )
						{
							query = "SELECT ";
							local_addReturnFields(token, query);
							query.append(" FROM ").append(dbName.data).append(".").append(tableName.data).append(" WHERE ").append(key.name).append(" = LAST_INSERT_ID()");
							int result = LINUXFUBAR mysql_query(mysql, query.c_str());
							if ( result == 0 )
							{
								MYSQL_RES * res = LINUXFUBAR mysql_store_result(mysql);
								int num = LINUXFUBAR mysql_num_fields(res);
								local_addColumnInfo( token, res, mysql, dbName.data, tableName.data );
								MYSQL_ROW row = LINUXFUBAR mysql_fetch_row(res);
								if ( row != NULL )
								{
									unsigned long * lengths = LINUXFUBAR mysql_fetch_lengths(res);
									lasso_addResultRow( token, (const char**)row, (unsigned int*)lengths, num );
									LINUXFUBAR mysql_free_result(res);
								}
							}
						}
					}
				}
				err = CheckError(token, mysql);
			}
			break;
		case datasourceUpdate:
			 {
				auto_lasso_value_t db, table;
				lasso_getDataSourceName(token, &db, NULL, NULL);
				lasso_getTableName(token, &table);
				
				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				string query;

				query.append("UPDATE ").append(db.data).append(".").append(table.data);

				local_addUpdateCriteria( token, query );
				if ( !local_addPrimaryKeySearch( token, query ) )
				{
					return osErrWebUpdateError;
				}
#ifdef DEBUG_SQL
				lasso_log(LOG_LEVEL_DETAIL, "MySQLDS made query: %s", query.c_str());
#endif
				if (LINUXFUBAR mysql_query(mysql, query.c_str()) == 0)
				{				
					if ( LINUXFUBAR mysql_affected_rows(mysql) == 0 )
					{
						lasso_setVariable( token, kRowCountVar, "0", 0 );
						int c;
						lasso_getMaxRows(token, &c);
						if ( c > 0 )
						{
							query = "SELECT ";
							local_addReturnFields(token, query);
							query.append(" FROM ").append(db.data).append(".").append(table.data);
							local_addPrimaryKeySearch( token, query );

							int result = LINUXFUBAR mysql_query(mysql, query.c_str());
							if ( result == 0 )
							{
								MYSQL_RES * res = LINUXFUBAR mysql_store_result(mysql);
								int num = LINUXFUBAR mysql_num_fields(res);
								local_addColumnInfo( token, res, mysql, db.data, table.data );
								MYSQL_ROW row = LINUXFUBAR mysql_fetch_row(res);
								if ( row != NULL )
								{
									unsigned long * lengths = LINUXFUBAR mysql_fetch_lengths(res);
									lasso_addResultRow( token, (const char**)row, (unsigned int*)lengths, num );
									LINUXFUBAR mysql_free_result(res);
								}
							}
						}
					}
					else
					{
						char rowcount[16] = "";      
						sprintf( rowcount, "%lu", (unsigned long) LINUXFUBAR mysql_affected_rows(mysql) );
						lasso_setVariable( token, kRowCountVar, rowcount, 0 );
					}
				}
				err = CheckError(token, mysql);
			}
			break;
		case datasourceDelete:
			 {
				auto_lasso_value_t db, table;

				lasso_getDataSourceName( token, &db, NULL, NULL );
				lasso_getTableName( token, &table );
					
				MYSQL * mysql = local_doConnect(token);
				if ( mysql == NULL )
					return osErrConnectionInvalid;

				string query;

				query.append("DELETE FROM ").append(db.data).append(".").append(table.data);
				if ( !local_addPrimaryKeySearch( token, query ) )
				{
					return osErrWebDeleteError;
				}
#ifdef DEBUG_SQL
				lasso_log(LOG_LEVEL_DETAIL, "MySQLDS made query: %s", query.c_str());
#endif
				if (LINUXFUBAR mysql_query(mysql, query.c_str()) == 0)
				{
					if ( LINUXFUBAR mysql_affected_rows(mysql) == 0 )
					{
						err = osErrWebDeleteError;
						lasso_setVariable( token, kRowCountVar, "0", 0 );
					}
					else
					{
						char rowcount[16] = "";      
						sprintf( rowcount, "%lu", (unsigned long)LINUXFUBAR mysql_affected_rows(mysql) );
						lasso_setVariable( token, kRowCountVar, rowcount, 0 );
					}			    
				}
				err = CheckError(token, mysql);
			}
			break;
		case datasourceInfo:
			 {
				auto_lasso_value_t dbName;
				auto_lasso_value_t tableName;

				lasso_getDataSourceName( token, &dbName, NULL, NULL );
				lasso_getTableName( token, &tableName );
				if ( tableName.data[0] != '\0' )
				{
					MYSQL * mysql = local_doConnect(token);
					if ( mysql == NULL )
						return osErrConnectionInvalid;
									
					char query[256] = "";
					/* no result set but we get the column info */
					sprintf(query, "SELECT * FROM %s.%s WHERE 1=0", dbName.data, tableName.data);

					if ( LINUXFUBAR mysql_query(mysql, query) == 0 )
					{
						MYSQL_RES * res = LINUXFUBAR mysql_store_result(mysql);
						
						local_addColumnInfo( token, res, mysql, dbName.data, tableName.data );
						
						LINUXFUBAR mysql_free_result(res);
					}
					err = CheckError(token, mysql);
				}
			}
			break;
		case datasourceExecSQL:
			if ( param != NULL && param->data != NULL )
			{				
				MYSQL * mysql = local_doConnect(token);
				printf("datasourceExecSQL: %s\n", LINUXFUBAR mysql_get_server_info(mysql));
				if ( mysql == NULL )
					return osErrConnectionInvalid;
				
				MYSQL_RES * res = NULL;
				auto_lasso_value_t dbName;
				lasso_getDataSourceName( token, &dbName, NULL, NULL );
				
				LINUXFUBAR mysql_select_db( mysql, dbName.data );	
				
				int max = param->dataSize, start=-1, finish=-1;
				while( err == osErrNoErr && FindStat(param->data, &start, &finish, max) )
				{
					if ( res )
					{
						LINUXFUBAR mysql_free_result(res);
						res = NULL;
					}
					if (LINUXFUBAR mysql_real_query( mysql, param->data + start, finish-start ) == 0)
						res = LINUXFUBAR mysql_store_result(mysql);
					else
						err = LINUXFUBAR mysql_errno(mysql);
				}
				if ( err == osErrNoErr )
				{
					lasso_setRowID( token, (unsigned long) LINUXFUBAR mysql_insert_id( mysql ) );

					char rowcount[16] = "";      
					sprintf( rowcount, "%lu", (unsigned long)LINUXFUBAR mysql_affected_rows(mysql) );
					lasso_setVariable( token, kRowCountVar, rowcount, 0 );

					if ( res != NULL )
					{
						local_addColumnInfo(token, res, mysql, "", "");

						int found = (int)LINUXFUBAR mysql_num_rows(res);
						int num = LINUXFUBAR mysql_num_fields(res);
						int skip, max;
						MYSQL_ROW row;

						lasso_getSkipRows( token, &skip );
						lasso_getMaxRows( token, &max );
						lasso_setNumRowsFound( token, found );
					
						if ( skip > 0 )
							LINUXFUBAR mysql_data_seek( res, skip );

						while( (row = LINUXFUBAR mysql_fetch_row(res)) != NULL && max--)
						{
							unsigned long * lengths = LINUXFUBAR mysql_fetch_lengths(res);
							lasso_addResultRow( token, (const char**)row, (unsigned int*)lengths, num );
						}
						LINUXFUBAR mysql_free_result(res);
					}
				}
				err = CheckError(token, mysql);
			}
			break;
		default:
			break;

	}
	return err;
}

struct string_ptr_pair_t
{
	string_ptr_pair_t( const std::string & str, const std::auto_ptr<std::string> & ptr)
	{
		first = str;
		second = const_cast<std::auto_ptr<std::string>&>(ptr);
	}
	string_ptr_pair_t( const string_ptr_pair_t & pr )
	{
		first = pr.first;
		second = const_cast<string_ptr_pair_t&>(pr).second;
	}
	string_ptr_pair_t(){}

	std::string first;
	std::auto_ptr<std::string> second;
};
typedef std::list<string_ptr_pair_t> consolidated_input_t;

inline void AddOrCat( const char * name, const char * value, int valueLen, consolidated_input_t * lst )
{
	for ( consolidated_input_t::iterator it = lst->begin(), end = lst->end(); it != end; ++it )
	{
		if ( (*it).first == name ) // it's in there
		{
			if ( value != NULL )
			{
				if ( (*it).second.get() == NULL )
					(*it).second.reset(new std::string);
				else
					(*it).second->append(",");
				(*it).second->append(value, valueLen);
			}
			return;
		}
	}
	// add it anew
	lst->push_back(string_ptr_pair_t(std::string(name), std::auto_ptr<std::string>(NULL)));
	if ( value != NULL )
		lst->back().second.reset(new std::string(value, valueLen));
}

inline void
ConsolidateInput( lasso_request_t token, consolidated_input_t * lst )
{
	int num,i;

	lasso_getInputColumnCount( token, &num );	
	for ( i = 0; i < num; ++i )
	{
		auto_lasso_value_t val;
		if ( lasso_getInputColumn( token, i, &val ) == osErrNoErr )
		{
			AddOrCat( val.name, val.data, val.dataSize, lst);
		}
	}
}

void local_addInsertCriteria( lasso_request_t token, string & query )
{
	int num, i;
	consolidated_input_t lst;
	ConsolidateInput(token, &lst);

	num = lst.size();
	if ( num > 0 )
	{
		query.append(" (");
		i = 0;
		{for ( consolidated_input_t::iterator it = lst.begin(), end = lst.end(); it != end; ++it )
		{
			query.append((*it).first);
			if ( i + 1 < num )
				query.append(",");
			++i;
		}}
		query.append(") VALUES (");
		i = 0;
		{for ( consolidated_input_t::iterator it = lst.begin(), end = lst.end(); it != end; ++it )
		{
			const char * str = (*it).second.get() ? (*it).second->c_str() : NULL;
			int len = (*it).second.get() ? (*it).second->length() : 0;
			local_addSearchValue( query, str, len, lpTypeNull);
			
			if ( i + 1 < num )
				query.append(",");
			++i;
		}}
		query.append(")");
	}	
}

void local_addUpdateCriteria( lasso_request_t token, string & to )
{
	consolidated_input_t lst;
	bool addComma = false;

	ConsolidateInput( token, &lst );

	if ( lst.size() > 0 )
	{
		to.append(" SET ");
		for ( consolidated_input_t::iterator it = lst.begin(), end = lst.end(); it != end; ++it )
		{
			if ( addComma )
				to.append(",");
			else
				addComma = true;

			to.append((*it).first);
			to.append(" = ");
			const char * str = (*it).second.get() ? (*it).second->c_str() : NULL;
			int len = (*it).second.get() ? (*it).second->length() : 0;
			local_addSearchValue( to, str, len, lpTypeNull );		
		}
	}
}

bool local_addPrimaryKeySearch( lasso_request_t token, string & to )
{
	auto_lasso_value_t key;
	
	if ( lasso_getPrimaryKeyColumn( token, &key ) == osErrNoErr )
	{
		if ( key.name != NULL && strlen(key.name) > 0 )
		{
			to.append(" WHERE ");
			to.append(key.name);
			to.append(" = ");
			if ( key.data != NULL && strlen(key.data) > 0 )
			{
				local_addSearchValue( to, key.data, key.dataSize, lpTypeNull );
				return true;
			}
		}
	}
	return false;
}
/*
+-------+--------------------------------+------+-----+---------+-------+
| Field | Type                           | Null | Key | Default | Extra |
+-------+--------------------------------+------+-----+---------+-------+
| Title | enum('Mr.','Mrs.','Ms.','Dr.') | YES  | MUL | NULL    |       |
+-------+--------------------------------+------+-----+---------+-------+
*/
void grabValueList( MYSQL * mysql, const char * col, const char * db, const char * table, char *** vals, int * count )
{
	std::string query;
	query.append("SHOW COLUMNS FROM ").append(table).append(" FROM ").append(db).append(" LIKE '").append(col).append("'");

	int result = LINUXFUBAR mysql_query(mysql, query.c_str());
	if ( result == 0 )
	{
		MYSQL_RES * res = LINUXFUBAR mysql_store_result(mysql);
		if ( res )
		{
			int num = LINUXFUBAR mysql_num_fields(res);
			MYSQL_ROW row = LINUXFUBAR mysql_fetch_row(res);
			if ( row != NULL && num >= 2 )
			{
				const char * def = row ? row[1] : NULL;

				if ( def && (strncmp("enum", def, 4) == 0 || strncmp("set", def, 3) == 0) )
				{
					if ( def[0] == 'e' )
						def += 5;
					else
						def += 4;

					*count = 1;
					for ( int tmp = 0; def[tmp] != '\0'; ++tmp )
					{
						if ( def[tmp] == ',' )
							++*count;
					}

					*vals = new char*[*count];

					for ( int x = 0; x < *count; ++x )
					{
						if ( *def == '\'' )
							++def;
						std::string str;
						while( *def != ',' && *def != '\'' )
						{
							str.append(def, 0, 1);
							++def;
						}
						if ( *def == '\'' )
							def += 2;
						else
							++def;

						(*vals)[x] = new char[str.length()+1];
						strcpy((*vals)[x], str.c_str());
					}
				}
			}
			LINUXFUBAR mysql_free_result(res);
		}
	}
}

void local_addColumnInfo( lasso_request_t token, MYSQL_RES * res, MYSQL * mysql, const char * db, const char * table )
{
	int numFields = LINUXFUBAR mysql_num_fields(res), i;
	MYSQL_FIELD * field = LINUXFUBAR mysql_fetch_fields(res);	

	for( i = 0; i < numFields; ++i )
	{
		LP_TypeDesc type;
		switch( field[i].type )
		{
			case FIELD_TYPE_TINY:
			case FIELD_TYPE_SHORT:
			case FIELD_TYPE_LONG:
			case FIELD_TYPE_INT24:
			case FIELD_TYPE_LONGLONG:
				type = lpTypeInteger;
				break;
			case FIELD_TYPE_DECIMAL:
			case FIELD_TYPE_FLOAT:
			case FIELD_TYPE_DOUBLE:
				type = lpTypeDecimal;
				break;
			case FIELD_TYPE_TIMESTAMP:
			case FIELD_TYPE_DATE:
			case FIELD_TYPE_TIME:
			case FIELD_TYPE_DATETIME:
			case FIELD_TYPE_YEAR:
				type = lpTypeDateTime; // ???
				break;
			default:
				type = lpTypeString;
				break;
		}		
		if ( (db != NULL && table != NULL && db[0] != '\0' && table[0] != '\0') && field[i].name && ((field[i].flags & ENUM_FLAG) != 0 || (field[i].flags & SET_FLAG) != 0) )
		{
			char ** vals = NULL;
			int count = 0;

			grabValueList(mysql, field[i].name, db, table, &vals, &count);
			lasso_addColumnInfo2( token, field[i].name, IS_NOT_NULL(field[i].flags) ? 0 : 1, type, kProtectionNone, (const char**)vals, count );

			for ( int o = 0; o < count; ++o )
				delete [] vals[o];
			delete [] vals;
		}
		else
			lasso_addColumnInfo( token, field[i].name, IS_NOT_NULL(field[i].flags) ? 0 : 1, type, kProtectionNone );
	}
}

void local_addReturnFields( lasso_request_t token, string & to )
{
	int num, i;
	lasso_getReturnColumnCount( token, &num );

	if ( num == 0 )
	{
		to.append("*");
	}
	else
	{
		for ( i = 0; i < num; ++i )
		{
			auto_lasso_value_t val;
			if ( lasso_getReturnColumn( token, i, &val ) == osErrNoErr )
			{
				if ( i > 0 )
					to.append(",");
				to.append(val.data);
			}
			else
				break;
		}
	}
}

#define ISOPBEGIN(X) (strcasecmp(X, kOpBegin) == 0 || strcasecmp(X, kOperatorBegin) == 0)
#define ISOPEND(X) (strcasecmp(X, kOpEnd) == 0 || strcasecmp(X, kOperatorEnd) == 0)

void local_addSearchCriteria( lasso_request_t token, string & to, LP_TypeDesc defaultOp, bool isFindAll )
{
	int num, i;
	bool useLimit = false;
	auto_lasso_value_t val;
	LP_TypeDesc op;

	// if a primary key column was supplied then just use it for the search
	lasso_getPrimaryKeyColumn( token, &val );
	if ( !isFindAll && val.name != NULL && strlen(val.name) > 0 && val.data != NULL && strlen(val.data) > 0)
	{
		to.append(" WHERE ").append(val.name).append(" = '").append(val.data).append("'");
	}
	else
	{
		lasso_getInputColumnCount( token, &num );

		if ( num > 0 )
		{		
			std::vector<auto_lasso_value_t> validValues;
			
			lasso_getLogicalOp( token, &op );
			auto_lasso_value_t tmpVal;
			tmpVal.name = "opbegin";

			if ( op == kLassoOR )
			{
				tmpVal.data = "or";
			}
			else
			{
				tmpVal.data = "and";
			}
			if (!isFindAll)
				validValues.push_back(tmpVal);
			
			for ( i = 0; i < num; ++i )
			{
				if ( lasso_getInputColumn( token, i, &val ) != osErrNoErr )
					return;
				if (val.name == NULL)
					continue;
				if ( ISOPEND(val.name) )
					val.data = "something";
				else if ( strcasecmp(val.name, "-uselimit") == 0 )
				{
					useLimit = true;
					continue;
				}
				else if ( val.name[0] == '-' && !ISOPBEGIN(val.name) )
				{
					continue;
				}
				// allow blank values
				if ( !isFindAll && ((val.data != NULL && strlen(val.data) > 0) || val.type == kLassoEquals || val.type == kLassoNOT) )
				{
					if ( i != 0 && ISOPEND(val.name) && ISOPBEGIN(validValues.back().name) )
					{
						validValues.pop_back();
					}
					else
						validValues.push_back(val);
				}
			}

			num = validValues.size();
			if ( num > 1 )
			{
				to.append(" WHERE ");

				std::vector<LP_TypeDesc> opStack;

				for ( i = 0; i < num; ++i )
				{
					auto_lasso_value_t curr = validValues[i];
					if ( ISOPBEGIN(curr.name) )
					{
						if ( strcasecmp(curr.data, "not") == 0 )
						{
							to.append("NOT (");
							opStack.push_back(kLassoNOT);
						}
						else
						{
							if ( strcasecmp(curr.data, "and") == 0 )
								opStack.push_back(kLassoAND);
							else if ( strcasecmp(curr.data, "or") == 0 )
								opStack.push_back(kLassoOR);
							else
								opStack.push_back(op);

							to.append("(");
						}
						continue;
					}
					else if ( ISOPEND(curr.name) )
					{
						to.append(")");
						opStack.pop_back();
					}
					else
					{
						if ( curr.type == kLassoInFullText )
							to.append("MATCH(").append(curr.name).append(")");
						else
							to.append(curr.name);
						
						if ( defaultOp != lpTypeNull ) // no default operator
						{
							curr.type = defaultOp;
						}

						local_addOperator( to, curr.type, curr.data, curr.dataSize );
						local_addSearchValue( to, curr.data, curr.dataSize, curr.type );
					}
					if ( (i + 1) < num && (!ISOPEND(validValues[i+1].name)) )
					{
						int num = opStack.size()-1;
						LP_TypeDesc lastOp = opStack[num--];
			
						while( num >= 0 && lastOp == kLassoNOT )
						{
							lastOp = opStack[num--];
						}

						if ( lastOp != kLassoAND )
							to.append(" OR ");
						else
							to.append(" AND ");
					}			
				}
				while ( opStack.size() > 0 )
				{
					to.append(")");
					opStack.pop_back();
				}
			}
		}
	}
	local_addSortOrder( token, to );
	if ( useLimit )
	{
		int max, skip;
		lasso_getSkipRows( token, &skip );
		lasso_getMaxRows( token, &max );

		if ( max == -1 )
			max = 2147483647;
		char tmp[256];
		sprintf(tmp, " LIMIT %d,%d", skip, max);
		to.append(tmp);
	}
}

void local_addSortOrder( lasso_request_t token, string & to )
{
	int num, i;
	std::vector<auto_lasso_value_t> sorts;
	auto_lasso_value_t val;

	lasso_getSortColumnCount( token, &num );
	for ( i = 0; i < num; ++i )
	{
		if ( lasso_getSortColumn( token, i, &val ) == osErrNoErr )
		{
			if ( val.data != NULL && strlen(val.data) > 0 )
				sorts.push_back(val);
		}
		else
			break;
	}
	num = sorts.size();
	if ( num > 0 )
	{
		to.append(" ORDER BY ");
		for ( i = 0; i < num; ++i )
		{
			to.append(sorts[i].data);
			if ( sorts[i].type == orderDescending )
				to.append(" DESC ");
			if ( i + 1 < num )
				to.append(",");
		}
	}
	auto_lasso_value_t test;
	if ( lasso_findInputColumn(token, "-sortrandom", &test) == osErrNoErr )
	{
		to.append(" ORDER BY RAND()");
	}
}

void local_addOperator( string & to, LP_TypeDesc op, const char * value, unsigned int size )
{
	switch( op )
	{
	case kLassoEquals:
		if ( value == NULL )
			to.append(" IS ");
		else
			to.append(" = ");
		break;
	case kLassoGreaterThan:
		to.append(" > ");
		break;
	case kLassoGreaterThanEquals:
		to.append(" >= ");
		break;
	case kLassoLessThan:
		to.append(" < ");
		break;
	case kLassoLessThanEquals:
		to.append(" <= ");
		break;
	case kLassoNOT:
		if ( value == NULL )
			to.append(" IS NOT ");
		else
			to.append(" <> ");
		break;
	case kLassoContains:
	case kLassoBeginsWith:
	case kLassoEndsWith:
		to.append(" LIKE ");
		break;
	case kLassoNotContains:
	case kLassoNotBeginsWith:
	case kLassoNotEndsWith:
		to.append(" NOT LIKE ");
		break;
	case kLassoInFullText:
		to.append(" AGAINST ");
		break;
	case kLassoInRegExp:
		to.append(" REGEXP ");
		break;
	case kLassoNotInRegExp:
		to.append(" NOT REGEXP ");
		break;
	case kLassoInList:
		to.append(" IN ");
		break;
	case kLassoNotInList:
		to.append(" NOT IN ");
		break;
	default:
		to.append(" = ");
		break;
	}
}

void local_addSearchValue( string & to, const char * value, unsigned int size, LP_TypeDesc op )
{
	if ( value == NULL )
	{
		to.append("NULL");
	}
	else if ( value[0] == '\0' )
	{
		to.append("''");
	}
	else
	{
		// values need to be encoded
		char * encoded = new char[size*2+1];

		// mysql_real_escape_string didn't seem to be exported in the mysqlk client I am using
		LINUXFUBAR mysql_escape_string(encoded, value, size);	
		
		if ( op == kLassoEndsWith || op == kLassoContains || op == kLassoNotEndsWith || op == kLassoNotContains || 
				  op == kLassoBeginsWith || op == kLassoContains || op == kLassoNotBeginsWith || op == kLassoNotContains )
		{
			to.append("'");
			if (op == kLassoEndsWith || op == kLassoContains || op == kLassoNotEndsWith || op == kLassoNotContains)
				to.append("%");
			to.append(encoded);
			if (op == kLassoBeginsWith || op == kLassoContains || op == kLassoNotBeginsWith || op == kLassoNotContains)
				to.append("%");
			to.append("'");
		}
		else
		{	
			if ( op == kLassoInFullText )
				to.append("(");
			to.append("'").append(encoded).append("'");
			if ( op == kLassoInFullText )
				to.append(")");
		}

		delete [] encoded;
	}
}
