import blueworld.lasso.*;
import java.sql.*;
import java.util.*;

class JDBC_MySQL extends LassoDataSource
{
	private static final String DRIVER = "org.gjt.mm.mysql.Driver";
	private static final String CONN_URL = "jdbc:mysql://localhost:3306/mysql?user=monty";
	
	private Connection conn = null;
	private String lastError = new String("No Error");
	private int lastErrorCode = 0;	

	public JDBC_MySQL()
	{
		try {
			Class.forName( DRIVER ).newInstance(); 
			conn = DriverManager.getConnection( CONN_URL );
			System.out.println("[JDBC_MySQL] Database connection established.");
		} catch ( Exception e ) {
			System.err.println("[JDBC_MySQL] Database connection failed: " + e.getMessage());
		}
	}
		
	public void destroy()
	{
		try {
			conn.close();
			conn = null;
		} catch ( Exception e ) { }
	}
	
	protected void setErrorInfo(SQLException e)
	{
		lastError = new String( e.getMessage() );
		lastErrorCode = e.getErrorCode();
	}
	
	protected int sqlToLassoType(int type)
	{
		int ret = 0;
		
		switch (type)
		{
			case Types.BIT:
			case Types.BIGINT: 
			case Types.INTEGER:
			case Types.NUMERIC:
				ret = LassoDataTypes.TYPE_LONG;
				break;
			case Types.DATE:
				ret = LassoDataTypes.TYPE_DATE;
				break;			
			case Types.DECIMAL:
			case Types.FLOAT:
			case Types.DOUBLE:
				ret = LassoDataTypes.TYPE_FLOAT;
				break;
			case Types.CHAR:
			case Types.LONGVARCHAR:
				ret = LassoDataTypes.TYPE_CHAR;
				break;			
			case Types.BINARY:
			case Types.LONGVARBINARY:
				ret = LassoDataTypes.TYPE_BLOB;
				break;		
			case Types.NULL:
			case Types.OTHER:
			default:
				ret = LassoDataTypes.TYPE_NONE;
		}
		return (ret);
	}
	
	public String getModuleName()
	{
		return new String("JDBC_MySQL");
	}
	
	public boolean getDataSourceExists(String name)
	{
		if ( conn == null )
			return (false);
	
		try {
			ResultSet rs = conn.getMetaData().getCatalogs();
			while ( rs.next() )	
			{
				if ( name.equalsIgnoreCase( rs.getString(1) ) )
					return true;
			}
			rs.close();		
		} catch ( SQLException e ) {
			setErrorInfo( e );
		}
		return (false);
	}
	
	public int getDataSourceCount()
	{		
		int count = 0;
		
		if ( conn != null ) {						
			try {
				ResultSet rs = conn.getMetaData().getCatalogs();
				while ( rs.next() )	
					++count;
				rs.close();
			} catch ( SQLException e ) {
				setErrorInfo( e );
			}
		}
		return (count);	
	}
	
	public String[] getDataSourceNames()
	{	
		Vector v = new Vector();
		
		if ( conn != null )
		{	
			try {
				ResultSet rs = conn.getMetaData().getCatalogs();
				while ( rs.next() )	
					v.addElement( rs.getString(1) );
				rs.close();
			} catch ( SQLException e ) {
				setErrorInfo( e );
			}
		}

		String[] ret = new String[ v.size() ];
		for (int i=0; i<ret.length; i++)
			ret[i] = (String)v.elementAt(i);

		return (ret);
	}
	
	public String[] getTableNames(String ds, LassoInputParams input)
	{
		Vector v = new Vector();
		
		if ( ds != null && ds.length() != 0)
		{	
			try {
				DatabaseMetaData meta = conn.getMetaData();
				ResultSet tables = meta.getTables(ds, "", "", null);				
				while ( tables.next() )
					v.addElement( tables.getString(3) );
			} catch ( SQLException e ) {
				setErrorInfo( e );
			}
		}
		
		String[] ret = new String[v.size()];
		for (int i=0; i<ret.length; i++)
			ret[i] = (String)v.elementAt(i);
			
		return (ret);
	}
	
	public int getLastErrorCode()
	{
		return lastErrorCode;
	}
	
	public String getLastErrorString()
	{
		return new String( lastError );
	}
	
	public int doWebSearch(LassoSessionInfo session)
	{	
		return 0;	
	}
	
	public int doWebAdd(LassoSessionInfo session)
	{
		return 0;
	}
	
	public int doWebDuplicate(LassoSessionInfo session)
	{
		return 0;
	}
	
	public int doWebUpdate(LassoSessionInfo session)
	{
		return 0;
	}
	
	public int doWebDelete(LassoSessionInfo session)
	{
		return 0;
	}
	
	public int doWebShow(LassoSessionInfo session)
	{
		LassoOutputParams params = session.outputParams();

		try {
			DatabaseMetaData meta = conn.getMetaData();
			ResultSet cols = meta.getColumns(session.inputParams().getDBName(), "", 
											 session.inputParams().getTableName(), "");			
			while ( cols.next() )
			{
				params.addFieldName( cols.getString(4) ); //"COLUMN_NAME"
				params.addRepeats(false);
				params.addRepeatSize(0);
				params.addNullOK( cols.getInt(11) == DatabaseMetaData.columnNullable );  //"NULLABLE"
				params.addDefaultType( sqlToLassoType(cols.getShort(5)) ); //"DATA_TYPE"
			}
			cols.close();
			
		} catch ( SQLException e ) {
			setErrorInfo( e );
			return (-1);
		}
		
		return 0;
	}
	
	public int doWebScripts(LassoSessionInfo session)
	{
		return 0;
	}
	
	public int doWebSQL(LassoSessionInfo session)
	{
		String sql = session.inputParams().getSqlStatement();
		
		if ( sql == null || sql.length() == 0 ) 
			return (-1);
		
		LassoOutputParams params = session.outputParams();	
		LassoResultSet results = params.getResultSet();
		results.reset();
		
		int rows = 0;
		int cols = 0;
		int rowNum = 0;
		int skip = session.inputParams().getSkipRecs();
		int max = session.inputParams().getMaxRecs();
		
		try {
			ResultSet rs = conn.createStatement().executeQuery( sql );
			ResultSetMetaData meta = rs.getMetaData();
			cols = meta.getColumnCount();
	
			for (int i=1; i<=cols; i++)
			{
				params.addFieldName( meta.getColumnName(i) );
				params.addRepeats(false);
				params.addRepeatSize(0);
				params.addNullOK( meta.isNullable(i) == DatabaseMetaData.columnNullable );
				params.addDefaultType( sqlToLassoType(meta.getColumnType(i)) );
			}
			
			while ( rs.next() && max > 0)	
			{
				while( rows++ < skip)
					continue;
				
				for (int i=0; i<cols; i++)
				{						
					LassoResultColumn col = new LassoResultColumn();
					col.addValue( rs.getString(i+1) );
					results.addColumn(rowNum, col);
				}
				rowNum++;
				--max;
			}
			
			rs.close();

			params.setNumFound(rowNum);
			params.setNumTotal(rows);
				
		} catch ( SQLException e ) {
			setErrorInfo( e );
			return (-1);
		}
		return (0);
	}	
}
