import java.io.*;
import java.util.*;
import java.util.zip.*;
import com.blueworld.lassopro.*;

import org.apache.xerces.parsers.*;//DOMParser;
import org.apache.xpath.*;//XPathAPI;
import org.apache.xml.utils.*;//TreeWalker;
import org.apache.xpath.objects.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.traversal.NodeIterator;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;

// Imported JAVA API for XML Parsing 1.0 classes
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

// Imported Serializer classes
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;


/**
   This class implements two new tags: "xml_transform", "xml_extract".
 */
public class XmlTransformTag extends LassoTagModule
{
	// default constructor
	public XmlTransformTag()
	{
		super();
	}

	// This method is called when the user invokes [xml_transform] tag
	public int myTransformFunc (LassoCall lasso, int action)
	{
		String syntaxErr = "Syntax error: USAGE: [xml_transform: -XML='data', -XSL='xsl']";

		int err;
		IntValue temp = new IntValue();

		if ( ( err = lasso.getTagParamCount( temp ) ) != ERR_NOERR )
			return err;

		if ( temp.intValue() != 2 )
		{
			lasso.setResultMessage( syntaxErr );
			lasso.setResultCode( LassoErrors.InvalidParameter );
			return LassoErrors.InvalidParameter;
		}

		try
		{
			String xmlData = "";
			String xslData = "";
			String transformedData = "";

			String xmlFile = null;

			LassoValue outValue = new LassoValue();

			
			// xmldata
			if ( ( err = lasso.findTagParam("-xml", outValue ) ) != ERR_NOERR )
			{
				lasso.setResultMessage( syntaxErr );
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}
			
			xmlData = outValue.data();

			if ( null == xmlData || "" == xmlData )
			{
				lasso.setResultMessage( "-XML cannont be blank" );
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			// xsldata
			if ( ( err = lasso.findTagParam("-xsl", outValue ) ) != ERR_NOERR )
			{
				lasso.setResultMessage( syntaxErr );
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}
			xslData = outValue.data();

			if ( null == xslData || "" == xslData )
			{
				lasso.setResultMessage( "-XSL cannont be blank" );
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			// get TransformerFactory object
			TransformerFactory tFactory = TransformerFactory.newInstance();

			// get Transformer object
			Transformer transformer = tFactory.newTransformer(
										new StreamSource(new StringReader(
																xslData)));

			StringWriter writer = new StringWriter();
			StreamResult result = new StreamResult(writer);

			// applying transform
			transformer.transform(
				new StreamSource(new StringReader(xmlData)), result);

			transformedData = writer.toString();

			LassoTypeRef outString = new LassoTypeRef();

			if ( ( err = lasso.typeAllocString ( outString, transformedData ) ) != ERR_NOERR )
				return err;

			if ( ( err = lasso.returnTagValue( outString ) ) != ERR_NOERR )
				return err;
		}
		catch(TransformerConfigurationException tce)
		{
			lasso.setResultMessage( tce.toString() );
			err = LassoErrors.UnknownError;
			lasso.setResultCode( err );
		}
		catch(TransformerException te)
		{
			lasso.setResultMessage( te.toString() );
			err = LassoErrors.UnknownError;
			lasso.setResultCode( err );
		}
		catch(Exception e)
		{
			lasso.setResultMessage( e.toString() );
			err = LassoErrors.UnknownError;
			lasso.setResultCode( err );
		}
		finally
		{
			return err;
		}
	}

	// This method is called when the user invokes [xml_extract] tag
	public int myExtractFunc (LassoCall lasso, int action)
	{
		int err;
		IntValue temp = new IntValue();
		String syntaxErr = "Syntax error: USAGE: [xml_extract: -xml='data' or -file='filepath', -XPath='path']";

		if ( ( err = lasso.getTagParamCount( temp ) ) != ERR_NOERR )
			return err;

		if ( temp.intValue() != 2 )
		{
			lasso.setResultMessage( syntaxErr );
			lasso.setResultCode( LassoErrors.InvalidParameter );
			return LassoErrors.InvalidParameter;
		}

		try
		{
			String xmlData = null;
			String xPath = null;
			String xmlFile = null;
			Vector xPathReturned = null;

			LassoValue outValue = new LassoValue();


			// xmlFile
			if ( ( err = lasso.findTagParam("-file", outValue ) ) != ERR_NOERR )
			{
				// xmldata
				if ( ( err = lasso.findTagParam("-xml", outValue ) ) != ERR_NOERR )
				{
					lasso.setResultMessage( syntaxErr );
					lasso.setResultCode( LassoErrors.InvalidParameter );
					return LassoErrors.InvalidParameter;
				}
				
				xmlData = outValue.data();

				if ( null == xmlData || "" == xmlData )
				{
					lasso.setResultMessage( "-XML cannont be blank" );
					lasso.setResultCode( LassoErrors.InvalidParameter );
					return LassoErrors.InvalidParameter;
				}
			}
			else
			{
				xmlFile = outValue.data();
				if ( null == xmlFile || "" == xmlFile )
				{
					lasso.setResultMessage( "-file cannont be blank" );
					lasso.setResultCode( LassoErrors.InvalidParameter );
					return LassoErrors.InvalidParameter;
				}
			}	

			// xpath
			if ( ( err = lasso.findTagParam("-xpath", outValue ) ) != ERR_NOERR )
			{
				lasso.setResultMessage( syntaxErr );
				lasso.setResultCode( err );
				return err;
			}
			xPath = outValue.data();

			if ( null == xPath || "" == xPath )
			{
				lasso.setResultMessage( "xPath cannont be blank" );
				lasso.setResultCode( LassoErrors.InvalidParameter );
				return LassoErrors.InvalidParameter;
			}

			// getting XmlXPath object and making request for XPath information
			//XmlXpath obj = new XmlXpath();
			

			// Set up a DOM tree to query.

			Document doc = null;
			DocumentBuilderFactory dfactory = DocumentBuilderFactory.newInstance();
			dfactory.setNamespaceAware(true);
			
			if(xmlFile != null)
			{
				xmlFile = internalToFullPath(lasso, xmlFile);
				
				if(xmlFile != null) //we got valid full file path
				{
					if(lasso.operationAllowed(LassoSecureOperations.FILE_READ, xmlFile) != ERR_NOERR)
						return LassoErrors.CouldNotReadFromFile;
				}
				else
				{
					return LassoErrors.InvalidPathname;
				}
				
							
				doc = dfactory.newDocumentBuilder().parse(new File(xmlFile));
			}
			else if(xmlData != null)
			{
				ByteArrayInputStream bis = new ByteArrayInputStream(xmlData.getBytes());
				InputSource in = new InputSource(bis);
				doc = dfactory.newDocumentBuilder().parse(in);
			}


//date = new Date();
//System.out.println("Completed File parse..." + date);

			CachedXPathAPI cxp = new CachedXPathAPI();			
			XObject xon = cxp.eval(doc, xPath);

			if(xon.getType() == XObject.CLASS_NUMBER)
			{
				double count = xon.num();
				
				LassoTypeRef ref = new LassoTypeRef();
				err  = lasso.typeAllocDecimal(ref, (double) count);
				if(err == ERR_NOERR)
					err = lasso.returnTagValue(ref);
				else
					return err;
				return 0;
			}
			else if(xon.getType() == XObject.CLASS_BOOLEAN)
			{
				boolean bResult = xon.bool();
				
				LassoTypeRef ref = new LassoTypeRef();
				err  = lasso.typeAllocBoolean(ref, bResult);
				if(err == ERR_NOERR)
					err = lasso.returnTagValue(ref);
				else
					return err;
				return 0;
			}
			
			
			///xPathReturned = getXPath(doc, xPath);
			xPathReturned = getXPath(xon.nodeset());
			
			LassoTypeRef mArray 	= new LassoTypeRef();
			LassoTypeRef initial[] 	= new LassoTypeRef[1];
			LassoTypeRef sref 		= new LassoTypeRef();
			
			err = lasso.typeAllocArray(mArray, initial );
			int size = xPathReturned.size();
			for(int k = 0; k < size; k++)
			{
				err = lasso.typeAllocString(sref,(String)xPathReturned.elementAt(k)); //use iteration here
				err = lasso.arraySetElement(mArray, k, sref); // adding the elements to the array
			}
			
			if ( ( err = lasso.returnTagValue( mArray ) ) != ERR_NOERR )
				return err;

		}
		catch(IOException ioe)
		{
			lasso.setResultMessage( ioe.toString() );
			err = LassoErrors.UnknownError;
			lasso.setResultCode( err );
		}
		catch(TransformerConfigurationException tce) {
			lasso.setResultMessage( tce.toString() );
			err = LassoErrors.UnknownError;
			lasso.setResultCode( err );
		}
		catch(TransformerException te) {
			lasso.setResultMessage( te.toString() );
			err = LassoErrors.UnknownError;
			lasso.setResultCode( err );
		}
		catch(Exception e) {
			lasso.setResultMessage( e.toString() );
			err = LassoErrors.UnknownError;
			lasso.setResultCode( err );
		}
		finally
		{
			return (err);
		}
	}


	// converts an internal path (full or relative) into a full platform specific path
	String internalToFullPath(LassoCall lasso, String inPath)
	{
		String qualifiedPath = null;
		String resolvedPath = null;
		qualifiedPath = lasso.fullyQualifyPath( inPath );
		resolvedPath = lasso.resolvePath( qualifiedPath );
		return lasso.getPlatformSpecificPath( resolvedPath );
	}


	
	// called automatically to register the tags
	public void registerLassoModule() {

		// registers xml_transform tag
		registerTagModule("XmlTransformTag", "xml_transform", "myTransformFunc", FLAG_INITIALIZER | FLAG_SUBSTITUTION, "To Transform XML");

		// registers xml_extract tag
		registerTagModule("XmlTransformTag", "xml_extract", "myExtractFunc", FLAG_INITIALIZER | FLAG_SUBSTITUTION, "To Extract XPath info");

	}

	/**
	   Utility for applying an XPath epxression to an xml file and returning information
	   about the execution of the XPath object and the nodes it finds.
	 */

	/** Process input args and execute the XPath.  */
	public Vector getXPath(/*Document doc, String xPath*/NodeIterator nl) throws Exception
	{

		// Set up an identity transformer to use as serializer.
		Transformer serializer = TransformerFactory.newInstance().newTransformer();
		serializer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");

		// Use the simple XPath API to select a nodeIterator.
		////NodeIterator nl = XPathAPI.selectNodeIterator(doc, xPath);

		// Serialize the found nodes to System.out.
		StringWriter writer = new StringWriter();
		StreamResult result = new StreamResult(writer);

		Node n;
		String prev = null;
		
		Vector vecStrings = new Vector();
				
		while ((n = nl.nextNode())!= null)
		{
			if (isTextNode(n))
			{
				// DOM may have more than one node corresponding to a
				// single XPath text node.  Coalesce all contiguous text nodes
				// at this level
				vecStrings.add(n.getNodeValue());

				for (Node nn = n.getNextSibling(); isTextNode(nn); nn = nn.getNextSibling())
				{
					vecStrings.add(nn.getNodeValue());
				}
			}
			else
			{
				serializer.transform(new DOMSource(n), result);

				String nodeVal = writer.toString();
				if(prev != null)
				{
					if(nodeVal.startsWith(prev))
					{
						String temp = nodeVal.substring(prev.length());
						prev = nodeVal;
						nodeVal = temp;
					}
				}
				else
					prev = nodeVal;
				vecStrings.add(nodeVal);
			}
		}

		return vecStrings;
	}

	/** Decide if the node is text, and so must be handled specially */
	boolean isTextNode(Node n)
	{
		if (n == null)
			return false;

		short nodeType = n.getNodeType();
		return nodeType == Node.CDATA_SECTION_NODE || nodeType == Node.TEXT_NODE;
	}


} // end class




