<?LassoScript

include:lassoapp_link:'../xml_support.lasso';
include:lassoapp_link:'xsd.lasso';

define_tag:'wsdl_getLocalWSDL', -description='Returns the WSDL describing the locally hosted operations.';

/define_tag;

define_tag:'wsdl_getServices', -required='xml', 
		-description='Returns an array of the service elements in the wsdl.';
	return: @(#xml->extract:"/*[local-name()='definitions']/*[local-name()='service']");
/define_tag;

define_tag:'wsdl_getServiceDocumentation', -required='srvc', -description='Returns the documentation element for the service.';
	local:'r' = @#srvc->(extract:"*[local-name()='documentation']/text()");
	if:#r && #r->size;
		return: #r->(get:1)->contents;
	/if;
	return:'';
/define_tag;

define_tag:'wsdl_getPortsForService', -required='srvc', 
		-description='Returns the ports for the given service.';
	return:@(#srvc->extract:"*[local-name()='port']");
/define_tag;

define_tag:'wsdl_getPortAddress', -required='port', -description='Returns the address location for the port.';
	local:'r'=@#port->(extract:"*[local-name()='address']/@location");
	if:#r->size;
		return:#r->(get:1)->contents;
	/if;
	return:'';
/define_tag;

define_tag:'wsdl_getBinding', -required='xml', -required='name',
		-description='Returns the binding for the given name.';
	
	local:'r'=@(#xml->(extract:"/*[local-name()='definitions']/*[local-name()='binding'][@name='" + #name + "']"));
	if:#r->size;
		return:@(#r->get:1);
	/if;
/define_tag;

define_tag:'wsdl_getBindingForOperation', -required='xml', -required='name';
	local:'str'= "/*[local-name()='definitions']/*[local-name()='binding']/*[local-name()='operation' and @name='";
	#str += #name;
	#str += "']/..";
	local:'r' = @(#xml->extract:#str);
	if:#r->size;
		return:#r->get:1;
	/if;
/define_tag;

define_tag:'wsdl_getMessage', -required='xml', -required='name',
		-description='Returns the message for the given name.';
	return:@(#xml->(extract:"/*[local-name()='definitions']/*[local-name()='message'][@name='" + #name + "']"))->get:1;
/define_tag;

define_tag:'wsdl_getPortType', -required='xml', -required='name',
		-description='Returns the portType for the given name.';
	return:@(#xml->(extract:"/*[local-name()='definitions']/*[local-name()='portType'][@name='" + #name + "']"))->get:1;
/define_tag;

define_tag:'wsdl_getBindingOperations', -required='binding', 
		-description='Returns the operations listed for the given binding.';
	return:@(#binding->extract:"*[local-name()='operation']");
/define_tag;

define_tag:'wsdl_getPortTypeOperation', -required='xml', -required='name';
	return:@(#xml->(extract:"/*[local-name()='definitions']/*[local-name()='portType']/*[local-name()='operation'][@name='" + #name + "']"))->get:1;
/define_tag;

define_tag:'wsdl_isSoapOperation', -required='op', -description='Checks the element name to see if it is a SOAP operation.';
	return:#op->name == (#soapns + 'operation');
/define_tag;

define_tag:'wsdl_listOperations', -required='xml';
	local:'r' = @(#xml->extract:"/*[local-name()='definitions']/*[local-name()='binding']/*[local-name()='operation']/@name");
	if:#r->size;
		local:'ary'=array;
		iterate:#r, local:'i';
			#ary->insert:(#i->contents);
		/iterate;
		return:#ary;
	/if;
/define_tag;

define_tag:'wsdl_getMessageTrioFromPortType', -required='portType', -required='opName';
	local:'str'="*[local-name()='operation' and @name='";
	#str += #opName;
	#str += "']/*[local-name()='input' or local-name()='output' or local-name()='fault']";
	local:'r'=@(#portType->extract:#str);
	if:#r->size;
		return:#r;
	/if;
/define_tag;

define_tag:'wsdl_getMessageNamed', -required='xml', -required='name';
	
	local:'str' = "/*[local-name()='definitions']/*[local-name()='message' and @name='";
	#str += xml_removeNamespace:#name;
	#str += "']";
	local:'r' = @(#xml->extract:#str);
	if:#r->size;
		return:#r->get:1;
	/if;
/define_tag;

define_tag:'wsdl_extract', -required='xml', -required='opName';
	local:'ret'=map;
	// find the definitions node
	// should only be one. if there are multiple, get only the first
	local:'defs'=(#xml->extract:"/*[local-name()='definitions']");
	if:#defs->size;
		#xml = #defs->get:1;
	/if;
	fail_if:((xml_removeNamespace:#xml->name) != 'definitions'), -1, 'The file does not appear to be a valid wsdl file. ' + #xml->name;
	
	// grab the namespaces
	local:'wsdlns'='';
	local:'soapns'='';
	local:'xsdns'='';
	local:'targetNs'=(xml_getAttribute:#xml, 'targetNamespace');	
	local:'tns'='';
	
	local:'namespaces'= @(#xml->namespaces);
	iterate:#namespaces, local:'curns';
		// don't process default namespace
		if:#curns->first; // default ns will have null prefix
			select:#curns->second;
				case:'http://schemas.xmlsoap.org/wsdl/';
					#wsdlns = #curns->first;
				case:'http://www.w3.org/2001/XMLSchema';
					#xsdns = #curns->first;
				case:'http://schemas.xmlsoap.org/wsdl/soap/';
					#soapns = #curns->first;
				case:#targetNs;
					#tns = #curns->first;
			/select;
		/if;
	/iterate;
	if:#soapns != '';
		#soapns += ':';
	/if;
	if:#tns != '';
		#tns += ':';
	/if;
	
	// find the binding for the spec'd operation
	local:'binding'= @(wsdl_getBindingForOperation:#xml, #opName);
	// find the portType for this binding
	local:'portType'= @(wsdl_getPortType:#xml, xml_removeNamespace:(xml_getAttribute:#binding, 'type'));
	// grab the input, output and fault nodes
	local:'iofNames' = @(wsdl_getMessageTrioFromPortType:#portType, #opName);
	
	local:'input', 'output', 'fault';
	
	iterate:#iofNames, local:'scratch';
		select:xml_removeNamespace:(#scratch->name);
			case:'input';
				#input = @(wsdl_getMessageNamed:#xml, (xml_getAttribute:#scratch, 'message'));
			case:'output';
				#output = @(wsdl_getMessageNamed:#xml, (xml_getAttribute:#scratch, 'message'));
			case:'fault';
				#fault = @(wsdl_getMessageNamed:#xml, (xml_getAttribute:#scratch, 'message'));
		/select;
	/iterate;
	
	local:'inputTypes'=array;
	local:'outputTypes'=array;
	
	// find the schema section(s)
	local:'schemas'=@(xml_getSchemas:#xml);
	
	define_tag:'getSchemaPart', -required='schemas', -required='typeName';
		iterate:#schemas, local:'i';
			local:'answer'= @(#i->extractOne:"*[@name='" + #typeName + "']");
			if:#answer->isa:'xml';
				return:@#answer;
			/if;
		/iterate;
	/define_tag;
	
	fail_if:#schemas->size == 0, -1, 'Could not find any valid schemas in the WSDL.';
	
	iterate:#input->children, #scratch;
		if:#scratch->name == 'part';
			local:'typeName' = xml_removeNamespace:(xml_getAttribute:#scratch, 'type');
			#inputTypes->insert:@(getSchemaPart:#schemas, #typename);
		/if;
	/iterate;
	
	iterate:#output->children, #scratch;
		if:#scratch->name == 'part';
			local:'typeName' = xml_removeNamespace:(xml_getAttribute:#scratch, 'type');
			#outputTypes->insert:@(getSchemaPart:#schemas, #typename);
		/if;		
	/iterate;
	
	#ret->insert:'operation'='opName';
	#ret->insert:'input'=@#inputTypes;
	#ret->insert:'output'=@#outputTypes;
	return:@#ret;
/define_tag;

//wsdl_extract:xml:include_url:'http://soap.amazon.com/schemas/AmazonWebServices.wsdl', 'ArtistSearchRequest';

?>