<?LassoScript

// set the global types spec
if:(global:'lp7_types.xsd')->type == 'null';
	global:'lp7_types.xsd' = xml:include:lassoapp_link:'XML/lp7types.xsd';
/if;

define_type:'prettyPrintingNSMap', 'map';
    define_tag:'onConvert';
        if (params->first == 'string');
            local:'ret'='';
            iterate:self, local:'i';
                #ret += '<font color="red">' #i->first '</font>';
                #ret += '<br/>\n';
                #ret += '<div class="typebox">' #i->second '</div>';
            /iterate;
            return:#ret;
        /if;
    /define_tag;
/define_type;

define_type:'prettyPrintingTypeMap', 'map';
    define_tag:'onConvert';
        if (params->first == 'string');
            local:'ret'='';
            iterate:self, local:'i';
                #ret += #i->second;
                #ret += '\n';
            /iterate;
            return:#ret;
        /if;
    /define_tag;
    
    define_tag:'findType', -required='ns', 
                -description='Returns a type, if found, given a <namespace>:<typename> string.';
        local:'keys'=#ns->split(':');
        local:'nsMap' = @self->find(keys->first);
        if (#nsMap->isa('map'));
            return(#nsMap->find(keys->second));
        /if;        
    /define_tag;
/define_type;

global:'XML_SCHEMA_URIS' = (:'http://www.w3.org/1999/XMLSchema', 'http://www.w3.org/2001/XMLSchema');
if: (global:'_xml_schema_cache')==null;
    (global:'_xml_schema_cache') = prettyPrintingNSMap;
/if;

define_type:'xsd_attribute';
    local:'name'='';
    local:'base'='';
    local:'use'='';
    local:'fixed'='';
    local:'default'=null;
    define_tag:'onConvert';
        if (params->first == 'string');
            return('<font color="green">'self->name'</font>=<font color="green">'self->base'</font>');
        /if;
    /define_tag;    
/define_type;

define_type:'xsd_ref';
    local:'string'='';
    define_tag:'onCreate';
        if (params->size);
            self->string = params->first;
        /if;
    /define_tag;
    define_tag:'onAssign';
        if:params->first->isa:'string';
            self->string = params->first;
            return:true;
        /if;
    /define_tag;
    define_tag:'onConvert';
        if (params->first == 'string');
            return:'<i>' self->string '</i><br/>\n';
        /if;
    /define_tag;
    define_tag:'namespace';
        local:'cpy'= self->string;
        #cpy->removetrailing(self->name);
        return:#cpy;
    /define_tag;
    define_type:'name';
        return:self->string->split(':')->last;    
    /define_type;
/define_type;

define_type:'xsd_type';
    local:'name'='';
    local:'namespace'='';
    local:'base'='';
    local:'attributes'=null;
    local:'content'=null;
    local:'contentBinder'='';
    local:'comment'='';

    define_tag:'setName', -required='xml', -required='tns', -required='name';
        if (#name !>> ':');
            self->name = #name;
            self->namespace = #tns;
        else;
            local:'splt'=#name->split(':');
            self->name = #splt->last;
            self->namespace = xsd_lookupNamespaceUri(#xml, #splt->first);
        /if;
    /define_tag;
    define_tag:'onConvert';
        if (params->first == 'string');
            local:'p'='<div class="type">';
            #p += '<font color="purple">'self->namespace'</font>:<font color="purple"><b>'self->name'</b></font><br/>\n';
            if (self->base->size);
                #p += self->base;
            else;
                #p += 'NO_BASE';
            /if;
            #p += '<br/>\n';
            if (self->attributes->size);
                iterate:self->attributes, local:'att';
                    #p += #att '<br/>\n';
                /iterate;
            /if;
            if (self->contentBinder);
                #p += self->contentBinder '<br/>';
            else;
                #p += 'NO_CONTENT<br/>';
            /if;
            if (self->content->size);
                #p += '<div class="typebox">';
                iterate:self->content, local:'c';
                    #p += #c;
                /iterate;
                #p += '</div>';
            /if;
            return:#p + '</div>';
        /if;
    /define_tag;
/define_type;

define_tag:'xsd_lookupNamespaceUri', -required='xml', -type='xml', -required='prefix', -type='string';
    return:#xml->extractOne('normalize-space(ancestor-or-self::*/namespace::*[local-name()=\'' #prefix '\'])');
/define_tag;

define_tag:'xsd_fullyQualifyName', -required='xml', -type='xml', 
        -required='tns', -type='string', -required='name', -type='string';
    if (#name !>> ':'); // easy
        return: #tns ':' #name;
    /if;
    local:'splt'=#name->split(':');
    return:#xml->extractOne('normalize-space(ancestor-or-self::*/namespace::*[local-name()=\'' 
                #splt->first 
                '\'])')
                ':' #splt->last;
/define_tag;

define_tag:'xsd_buildType', -required='xml', -type='xml', -required='tns', -type='string';
    select:#xml->name;
        case:'element';
            return:xsd_processElement(#xml, #tns);
        case:'complexType';
            return:xsd_processComplexType(#xml, #tns);
        case:'simpleType';
            return:xsd_processSimpleType(#xml, #tns);
        case:'import';
            return:xsd_processImport(#xml, #tns);
        case;
            local:'newt'=xsd_type;
            select:#xml->name;
                case:'choice';
                    xsd_continueChoice(#xml, #tns, #newt);
                case:'sequence';
                    xsd_continueSequence(#xml, #tns, #newt);
                case:'all';
                    xsd_continueAll(#xml, #tns, #newt);
                case;
                    log_warning:'Unhandled XMLSchema node: ' #xml->path;    
            /select;
            return:#newt;
    /select;
/define_tag;

define_tag:'xsd_continueAttribute', -required='xml', -type='xml', 
                                        -required='tns', -type='string', 
                                        -required='type', -type='xsd_type';
    local('attr' = xsd_attribute);
    local('base' = #xml->extract('normalize-space(@type)');
    #attr->base = xsd_fullyQualifyName(#xml, #tns, #base);
    #attr->name = #xml->extract('normalize-space(@name)');
    #attr->use = #xml->extract('normalize-space(@use)');
    #attr->default = #xml->extract('normalize-space(@default)');
    #attr->fixed = #xml->extract('normalize-space(@fixed)');
    if (!(#type->attributes->isa:'array'));
        #type->attributes = array;
    /if;
    #type->attributes->insert(@#attr);
/define_tag;                                        

define_tag:'xsd_continueExtension', -required='xml', -type='xml', 
                                        -required='tns', -type='string', 
                                        -required='type', -type='xsd_type';
    local:'base' = #xml->extract('normalize-space(@base)');
    if (!#base);
        log_warning:'Missing base attribute: ' #xml->path;
    else;
        #type->base = xsd_fullyQualifyName(#xml, #tns, #base); // split namespace
    /if;
    iterate:#xml->extract("*"), local:'d';
        select:#d->name;
            case:'attribute';
                xsd_continueAttribute(#d, #tns, #type);
            case;
                log_warning:'Not understood: ' #d->path;
        /select;
    /iterate;
/define_tag;                                        

define_tag:'xsd_continueRestriction', -required='xml', -type='xml', 
                                        -required='tns', -type='string', 
                                        -required='type', -type='xsd_type';
    local('base' = #xml->extract('normalize-space(@base)'));
    if (#base->size);
        #type->base = xsd_fullyQualifyName(#xml, #tns, #base);
    /if;
    
/define_tag;

define_tag:'xsd_continueComplexContent', -required='xml', -type='xml', 
                                        -required='tns', -type='string', 
                                        -required='type', -type='xsd_type';
    iterate:#xml->extract("*"), local:'d';
        select:#d->name;
            case:'extension';
                xsd_continueExtension(#d, #tns, #type);
            case:'restriction';
                xsd_continueRestriction(#d, #tns, #type);
            case;
                log_warning:'Not understood: ' #d->path;
        /select;
    /iterate;
/define_tag;

define_tag:'xsd_continueSequence', -required='xml', -type='xml', 
                                        -required='tns', -type='string', 
                                        -required='type', -type='xsd_type';
    #type->contentBinder += 'sequence:';
    #type->content = array;

    local:'lst' = #xml->extract("*");
    iterate: #lst, local:'d';
        select: #d->name;
//            case:'choice';
//                xsd_continueChoice(#d, #tns, #type);
            case;
                local:'new'=xsd_buildType(#d, #tns);
                if (#new->isa('xsd_type') || #new->isa('xsd_ref'));
                    #type->content->insert(#new);
                /if;
        /select;
    /iterate;
/define_tag;

define_tag:'xsd_continueChoice', -required='xml', -type='xml', 
                                        -required='tns', -type='string', 
                                        -required='type', -type='xsd_type';
    #type->contentBinder += 'choice:';
    #type->content = array;
    iterate:#xml->extract("*"), local:'d';
        local:'new'=xsd_buildType(#d, #tns);
        if (#new->isa('xsd_type') || #new->isa('xsd_ref'));
            #type->content->insert(#new);
        /if;
    /iterate;
/define_tag;

define_tag:'xsd_continueAll', -required='xml', -type='xml', 
                                        -required='tns', -type='string', 
                                        -required='type', -type='xsd_type';
    #type->contentBinder += 'all:';
    #type->content = array;
    iterate:#xml->extract("*"), local:'d';
        local:'new'=xsd_buildType(#d, #tns);
        if (#new->isa('xsd_type') || #new->isa('xsd_ref'));
            #type->content->insert(#new);
        /if;
    /iterate;
/define_tag;

define_tag:'xsd_continueComplexType', -required='xml', -type='xml', 
                                        -required='tns', -type='string', 
                                        -required='type', -type='xsd_type';
    local('name' = #xml->extractOne('normalize-space(@name)'));
    if (#name->size);
        #type->setName(#xml, #tns, #name);        
    /if;
    iterate:#xml->extract("*"), local:'d';
        select:#d->name;
            case:'complexContent';
                xsd_continueComplexContent(#d, #tns, #type);
            case:'all';
                xsd_continueAll(#d, #tns, #type);
            case:'sequence';
                xsd_continueSequence(#d, #tns, #type);
            case:'attribute';
                xsd_continueAttribute(#d, #tns, #type);
            case:'choice';
                xsd_continueChoice(#d, #tns, #type);
            case;
                log_warning:'Not understood: ' #d->path;
        /select;
    /iterate;
    
/define_tag;

define_tag:'xsd_processComplexType', -required='xml', -type='xml', -required='tns', -type='string';
    local:'t'=xsd_type;
    xsd_continueComplexType(#xml, #tns, #t);
    return:#t;
/define_tag;

define_tag:'xsd_processSimpleType', -required='xml', -type='xml', -required='tns', -type='string';
    local('name' = #xml->extractOne('normalize-space(@name)'));
    
/define_tag;

define_tag:'xsd_processElement', -required='xml', -type='xml', -required='tns', -type='string';
    local('name' = #xml->extractOne('normalize-space(@name)'),
            'type' = #xml->extractOne('normalize-space(@type)'));

	if(#type->size == 0);
	   #type = #xml->extractOne('normalize-space(@ref)');
	   if (#type->size);
	       // this is a ref
	       // not a named type on its own
	       if (#type !>> ':');
	           #type = #tns ':' #type;
	       /if;
//	       log_detail:'ref type: ' #type;
	       return:xsd_ref:#type;
	   /if;
	/if;
    local:'element' = xsd_type;
	if (#name);
        #element->name = #name;
    /if;
    #element->namespace = #tns;
	if:#type->size > 0;
        #element->base = xsd_fullyQualifyName(#xml, #tns, #type);
	/if;
	
    iterate:#xml->extract('*'), local:'d';
        select:#d->name;
            case:'complexType';
                xsd_continueComplexType(#d, #tns, #element);
            case;
                log_warning:'Not understood: ' #d->path;
        /select;
    /iterate;
    return:#element;
/define_tag;

define_tag:'xsd_processImport', -required='xml', -type='xml', -required='tns', -type='string';
    return:null;
/define_tag;

define_tag:'xsd_processSchema', -required='xml', -required='tns';
    local:'tnsmap'=null;
    
    $_xml_schema_cache->insert(#tns=prettyPrintingTypeMap);
    #tnsmap = @$_xml_schema_cache->find(#tns);

    iterate:#xml->extract("*"), local('type');
        if ( $XML_SCHEMA_URIS >> #type->extract('namespace-uri()'));   
            local:'t' = xsd_buildType(#type, #tns);
            if (#t->isa('xsd_type') || #t->isa('xsd_ref'));
                #tnsmap->insert( #t->name = #t );
            /if;
        /if;
    /iterate;
/define_tag;

define_tag:'xsd_loadSchema', -required='xml', -type='xml', 
        -description='Loads and stores information about all contained types.';

    local:'schemas' = #xml->extract("//*[local-name()='schema']"); /* */

    iterate:#schemas, local('schema');
        // check its namespace
        local:'tns' = #schema->extractOne('namespace-uri()');

        if ($XML_SCHEMA_URIS >> #tns);
            local:'tns' = #schema->extractOne('normalize-space(@targetNamespace)');
            xsd_processSchema(#schema, #tns);
        /if;
    /iterate;
/define_tag;

define_tag:'xsd_getSchemaType', -required='for', -description='Accepts a Lasso type name and returns the name for it in the lp7_types.xsd';
	select:#for;
		case: 'null';
			return:"nullType";	
		case:'string';
			return:"stringType";		
		case:'integer';
			return:"integerType";			
		case:'decimal';
			return:"decimalType";
		case:'bytes';
			return:'bytesType';					
		case:'boolean';			
			return:"booleanType";	
		case:'pair';
			return:"pairType";		
		case:'array';
			return:"arrayType";			
		case:'map';
			return:"mapType";				
		case:'tag';
			return:"customTagType";
		case;
			return:'customType';
	/select;
	fail:-1, 'Unsupported type: ' #for;
/define_tag;

define_tag:'xsd_deserialize', -required='thedata', -type='xml', -required='type', -type='string', 
                    -priority='low', -returntype='integer', 
                    -criteria=((:'int', 'integer', 'positiveInteger', 'negativeInteger', 'nonPositiveInteger', 'nonNegativeInteger') >> #type);
    return:integer(string(#thedata->contents));
/define_tag;

define_tag:'xsd_deserialize', -required='thedata', -type='xml', -required='type', -type='string', 
                    -priority='low', -returntype='double', 
                    -criteria=((:'double', 'float') >> #type);
    return:double(string(#thedata->contents));
/define_tag;

define_tag:'xsd_deserialize', -required='thedata', -type='xml', -required='type', -type='string', 
                    -priority='low', -returntype='string',
                    -criteria=(#type == 'string');
    return:string(#thedata->contents);
/define_tag;

define_tag:'xsd_deserialize', -required='thedata', -type='xml', -required='type', -type='string', 
                    -priority='low';
    return:#thedata;
/define_tag;

?>