<?lassoscript

{
	//
	// Anonymous User
	//
	// This section will fix a modified anonymous user, or will replace it if it is
	// missing.  Afterwards, it will flush the security cache to enable the changes.

	inline:
		-database = 'lasso_internal',
		-sql = 'SELECT id, name, password, enabled FROM security_users WHERE id = 1';

		if: (found_count) == 0;
		
			// anonymous user was deleted
			log_critical: 'Adding missing anonymous user';

			inline:
				-database = 'lasso_internal',
				-sql = 'INSERT INTO security_users (id, id_site, name, password, created, enabled, comment) VALUES (1, 1, \'ANONYMOUS\', \'\', \'' + (date)->(format: '%Y-%m-%d %H:%M:%S') + '\', \'Y\', \'Any user without an explicit username and password.\')';
			/inline;

		else: ((field: 'name') != 'ANONYMOUS') || ((string: (field: 'password'))->(length) > 0) || ((field: 'enabled') != 'Y');

			// Anonymous user has been modified, so we are fixing it here.
			log_critical: 'Repairing anonymous user';

			inline:
				-database = 'lasso_internal',
				-sql = 'update security_users set name = \'ANONYMOUS\', password = \'\', enabled = \'Y\' where id = 1';
			/inline;

		/if;

	/inline;
	
	//
	// -SQL Permissions
	//
	// This section populates the 'Q' permissions stored in security_group_host_map
	// to all of the databases and tables in security_group_database_map.  In addition,
	// Q permissions are cleansed from security_group_table_map and inserted into the
	// AnyTable entry for each database that has Q permission.
		
	// Check for the security_group_host_map and create it if it doesn't exist.
	Local: 'table_exists' = false;
	Inline: -Database='lasso_internal', -SQL='SHOW COLUMNS FROM security_group_host_map';
		if: (found_count) > 0;
			#table_exists = true;
		/if;
	/inline;

	if: #table_exists == false;
		log_critical: 'Creating host permission table';
		// Create table if it doesn't exist already
		Inline: -Database='lasso_internal',
				-SQL='CREATE TABLE IF NOT EXISTS security_group_host_map ' +
				'(' +
					'id INT PRIMARY KEY AUTO_INCREMENT,' +
					'id_host INT(11) DEFAULT 0,' +
					'id_group INT(11) DEFAULT 0,' +
					'allow SET("Q"),' +
					'deny SET("Q"),' +
					'modified TIMESTAMP,' +
					'INDEX (id_host),' +
					'INDEX (id_group)' +
				')', 
				-maxrecords='all';
		/Inline;		
				
		// Populate security_group_host_map by inserting Q for any host that has Q permission for any databases
		local: 'allow_array' = (array);
		local: 'deny_array' = (array);
		inline: -Database='lasso_internal', 
				-SQL='SELECT DISTINCT map.allow,map.deny,map.id_group,dbs.id_host FROM security_group_db_map AS map LEFT JOIN security_datasource_databases AS dbs ON map.id_database=dbs.id WHERE ((map.allow LIKE "%Q%") OR (map.deny LIKE "%Q%")) AND (map.id_database <> 1)', 
				-maxrecords='all';
			records;
				local: 'temp' = (array: (field: 'id_host'), (field: 'id_group'), 'Q');
				if: ((string: (field: 'allow')) >> 'Q') && (#allow_array !>> #temp);
					#allow_array->(insert: #temp);
				/if;
				if: ((string: (field: 'deny')) >> 'Q') && (#deny_array !>> #temp);
					#deny_array->(insert: #temp);
				/if;
			/records;
		/inline;

		local: 'insert_array' = (array);
		iterate: #allow_array, (local: 'temp');
			if: (#deny_array >> #temp);
				#temp->(insert: 'Q');
				#deny_array->(removeall: #temp);
			else;
				#temp->(insert: '');
			/if;
			#insert_array->(insert: '"' + #temp->(join: '","') + '"');
		/iterate;
		iterate: #allow_array, (local: 'temp');
			#temp->(insert: '', 3);
			#insert_array->(insert: '"' + #temp->(join: '","') + '"');
		/iterate;
		
		if: #insert_array->size > 0;
			log_critical: 'Initializing host permission table';
			inline: -database='lasso_internal', 
					-SQL='INSERT INTO security_group_host_map (id_host,id_group,allow,deny) VALUES (' + #insert_array->(join: '),(') + ')',
					-maxrecords='all';
			/inline;
		/if;
	/if;
	
	// Remove Q permission from all databases in security_group_database_map except All Databases entries.
	inline: -database='lasso_internal', 
			-SQL='UPDATE security_group_db_map SET allow=REPLACE(allow,"Q",""), deny=REPLACE(deny,"Q","") WHERE (id_database <> 1)', 
			-maxrecords='all';
	/inline;

	// Remove Q permission from all tables in security_group_table_map except All Tables entries.
	inline: -database='lasso_internal', 
			-SQL='UPDATE security_group_table_map SET allow=REPLACE(allow,"Q",""), deny=REPLACE(deny,"Q","") WHERE (id_table <> 1)', 
			-maxrecords='all';
	/inline;

	// Populate security_group_database_map based on the preferences in the host map.
	inline: -findall, 
			-database='lasso_internal', 
			-table='security_group_host_map', 
			-keyfield='id', 
			-maxrecords='all';
		local: 'host_map' = (map);
		records;
			if: (#host_map >> (field: 'id_host'));
				local: 'database_list' = #host_map->(find: (field: 'id_host'));
			else;
				local: 'database_list' = (array);
				inline: -search,
						-database='lasso_internal',
						-table='security_datasource_databases',
						-keyfield='id',
						-op='eq', 'id_host' = (field: 'id_host'),
						-maxrecords='all',
						-returnfield='id';
					records;
						#database_list->(insert: (field: 'id'));
					/records;
				/inline;
				#host_map->(insert: (field: 'id_host') = #database_list);
			/if;		

			local: 'need_allow' = ((string: (field: 'allow')) >> 'Q');
			local: 'need_deny' = ((string: (field: 'deny')) >> 'Q');
			local: 'update_list' = (array);
			
			if: #need_allow || #need_deny;
				log_critical: 'Updating host permissions for host ' + (field: 'id_host') ' and group ' + (field: 'id_group');
				if: #database_list->size > 0;
					local: 'sql' = 'SELECT id,allow,deny,id_database FROM security_group_db_map WHERE id_group=' + (field: 'id_group') + ' AND id_database in (' + #database_list->(join: ',') + ')';
					inline: -database='lasso_internal', -sql=#sql, -maxrecords='all';
						records;
							#update_list->(insert: (field: 'id'));
							#database_list->(removeall: (field: 'id_database'));		
							/records;
					/inline;
				/if;
				if: #update_list->size > 0;
					if: #need_allow && #need_deny;
						local: 'sql' = 'UPDATE security_group_db_map SET allow=CONCAT(allow,",Q"), deny=CONCAT(deny,",Q") WHERE id IN (' + #update_list->(join: ',') + ')';
					else: #need_allow;
						local: 'sql' = 'UPDATE security_group_db_map SET allow=CONCAT(allow,",Q") WHERE id IN (' + #update_list->(join: ',') + ')';
					else: #need_deny;
						local: 'sql' = 'UPDATE security_group_db_map SET deny=CONCAT(deny,",Q") WHERE id IN (' + #update_list->(join: ',') + ')';
					/if;				
					inline: -database='lasso_internal', -sql=#sql, -maxrecords='all';
					/inline;						
				/if;
				iterate: #database_list, (local: 'temp');
					if: #need_allow && #need_deny;
						local: 'sql' = 'INSERT INTO security_group_db_map (id_group,id_database,allow,deny) VALUES (' + (field: 'id_group') + ',' + #temp + ',"Q","Q")';
					else: #need_allow;
						local: 'sql' = 'INSERT INTO security_group_db_map (id_group,id_database,allow,deny) VALUES (' + (field: 'id_group') + ',' + #temp + ',"Q","")';
					else: #need_deny;
						local: 'sql' = 'INSERT INTO security_group_db_map (id_group,id_database,allow,deny) VALUES (' + (field: 'id_group') + ',' + #temp + ',"","Q")';
					/if;
					inline: -database='lasso_internal', -sql=#sql, -maxrecords='all';
					/inline;						
				/iterate;
			/if;
			

		/records;
	/inline;
	
	// Add All Tables entries for all groups 
	inline: -findall, 
			-database='lasso_internal', 
			-table='security_groups', 
			-keyfield='id', 
			-maxrecords='all';
		records;
			// All Tables
			local: 'id_group' = (field: 'id');
			local: 'sql' = 'SELECT id FROM security_group_table_map WHERE id_group=' + (local: 'id_group') + ' AND id_table=1';
			inline: -database='lasso_internal', -sql=#sql, -maxrecords='all';
				if: (found_count > 0);
					local: 'sql' = 'UPDATE security_group_table_map SET allow=CONCAT(allow,",Q") WHERE id=' + (field: 'id');
				else;
					local: 'sql' = 'INSERT INTO security_group_table_map (id_group,id_table,allow,deny) VALUES (' + (local: 'id_group') + ',1,"Q","")';
				/if;
				inline: -database='lasso_internal', -sql=#sql, -maxrecords='all';
				/inline;
			/inline;
		/records;
	/inline;
	
	// Clean up any empty entries.
	inline: -database='lasso_internal', 
			-SQL='DELETE FROM security_group_host_map WHERE ((allow = "") OR (allow IS NULL)) AND ((deny = "") OR (deny IS NULL))',
			-maxrecords='all';
	/inline;
	inline: -database='lasso_internal', 
			-SQL='DELETE FROM security_group_db_map WHERE ((allow = "") OR (allow IS NULL)) AND ((deny = "") OR (deny IS NULL))',
			-maxrecords='all';
	/inline;
	inline: -database='lasso_internal', 
			-SQL='DELETE FROM security_group_table_map WHERE ((allow = "") OR (allow IS NULL)) AND ((deny = "") OR (deny IS NULL))',
			-maxrecords='all';
	/inline;

	//
	// Tag Table Index
	//
	// This section adds indexing to the security_group_tag_map table.
	//

	local: 'index' = (array);

	inline:
		-database = 'lasso_internal',
		-sql = 'SHOW INDEX FROM security_group_tag_map';

		records;
			#index->(insert: (field: 'key_name'));
		/records;

	/inline;

	local: 'sql' = (string);

	if: #index->(find: 'id_tag')->size == 0;
		#sql += 'ADD INDEX id_tag(id_tag)';
	/if;

	if: #index->(find: 'modified')->size == 0;
		if: #sql->length > 0;
			#sql += ', ';
		/if;

		#sql += 'ADD INDEX modified(modified)';
	/if;

	if: #sql->length > 0;
		#sql = 'ALTER TABLE security_group_tag_map ' + #sql;

		inline:
			-database='lasso_internal',
			-sql=#sql,
			-maxrecords='all';
		/inline;
	/if;

	//
	// Default File Extensions
	//
	
	inline: -search,
			-database='lasso_internal',
			-table='global_prefs',
			-maxrecords='1',
			-returnfield='data',
			-op='eq', 'store_key'='lasso_file_extensions';
		if: (found_count == 0);
			
			local: 'default_extensions' = (array: '.lasso','.lassoapp','.txt','.text','.htm','.html','.inc','.uld','.xml','.pdf','.jpg','.gif','.png','.bmp','.psd','.tif','.rgb','.cmyk');
			#default_extensions->sort;
			
			inline: -add,
					-database='lasso_internal',
					-table='global_prefs',
					-keyfield='store_key',
					'store_key'='lasso_file_extensions',
					'data'=#default_extensions->(join: ' ');
			/inline;
		/if;
	/inline;
	
	//
	// Refresh Lasso Security
	//
	
	_security_flushcaches;

}->run;

?>