<?LassoScript

	// Link Tags
	//
	// [Link_Params] normalizes a parameter array
	// [Link] ... [/Link] generates anchor tags with a link
	// [Link_URL] converts a parameter array into a URL
	// [Link_SetFormat] sets standard parameters which all link tags will use
	//
	// All link tags share a common set of parameters
	//
	// Array parameters are merged into the parameter array
	// Keyword/Value parameters are treated as command tags
	// Name/Value parameters are treated as action parameters
	// -No(Name) parameters remove a command tag from the output (e.g. -NoDatabase, -NoMaxRecords)
	// -NoSort removes all sort parameters
	// -NoAction removes the commands tags that specify actions (e.g. -Add, -Update)
	// -NoClassic removes the commands required by classic (e.g. -Database, -Table, -KeyField, Actions)
	// -No.(Name) parameters remove name/value parameters from the output (e.g. -No.First_Name, -No.Last_Name)
	// -NoParams removes all name/value parameters (previously -NoSearch)
	// -NoToken.(Name) parameters remove a particular token from the output (e.g. -NoToken.Token_Name)
	// -NoTokens removes all tokens
	//
	// The response is a special case.  It is set to [Reponse_FilePath] by default.
	// The response is always used as the base of the URL even if specified as -Response.	
	//
	// The -ReturnField, -SQL, -Username, and -Password parameters are never returned by the link tags.

	Define_Tag: 'Link_Params', -Priority='Replace';

		Local: 'Link_Action' = '-Nothing';
		Local: 'Link_Array' = (Array);
		Local: 'Link_Map' = (Map);
		
		Local: 'Filter_Global' = (Array: '-SQL', '-Username', '-Password', -ReturnField);
		Local: 'Filter_Multi' = (Array: '-Required', '-Op', '-OpEnd', '-OpBegin', '-SortField', '-SortOrder');
		Local: 'Filter_Action' = (Array: '-Add', '-Delete', '-Duplicate', '-FindAll', '-Nothing', '-Search', '-Show', '-Update');
		Local: 'Filter_Classic' = (Array: '-Database', '-Table', '-KeyField', -Schema);
		
		Local: 'Equivalence' = (Map:
			-add=-Add,
			-clientusername=-Username,
			-clientpassword=-Password,
			-datasource=-Database,
			-database=-Database,
			-delete=-Delete,
			-dupliate=-Duplicate,
			-findall=-FindAll,
			-keycolumn=-KeyField,
			-keyfield=-KeyField,
			-keyvalue=-KeyValue,
			-layout=-Table,
			-logicaloperator=-OperatorLogical,
			-logicaloper=-OperatorLogical,
			-maxrecords=-MaxRecords,
			-maxrows=-MaxRecords,
			-nothing=-Nothing,
			-op=-Op,
			-opbegin=-OpBegin,
			-opend=-OpEnd,
			-oper=-Op,
			-operator=-Op,
			-operatorbegin=-OpBegin,
			-operatorend=-OpEnd,
			-operatorlogical=-OperatorLogical,
			-oplogical=-OperatorLogical,
			-password=-Password,
			-recid=-KeyValue,
			-recordid=-KeyValue,
			-req=-Required,
			-required=-Required,
			-returncolumn=-ReturnField,
			-returnfield=-ReturnField,
			-schema=-Schema,
			-search=-Search,
			-show=-Show,
			-skiprecords=-SkipRecords,
			-skiprows=-SkipRecords,
			-sortcolumn=-SortField,
			-sortfield=-SortField,
			-sortorder=-SortOrder,
			-table=-Table,
			-update=-Update,
			-username=-Username,
		);

		if: !(local_defined: 'param_array');
			Local: 'param_array' = (Params);
		/if;

		if: (var_defined: '_link_format') && ($_link_format->(isa: 'array')) && ($_link_format->size > 0);
			#param_array->(merge: $_link_format);
		/if;

		Local: 'count' = 1;
		While: #count <= #param_array->Size;
			Local: 'temp' = @(#param_array->(Get: #Count));
			If: #temp->(IsA: 'array');
				#param_array->(Remove: #count);
				#param_array->(Merge: #temp, #count);
				Local: 'temp' = @(#param_array->(Get: #count));
			/if;
			
			if: #temp->(IsA: 'pair');
				if: #temp->first->(isa: 'string') && #temp->first->(beginswith: '-');
					local: 'name' = #Equivalence->(Find: #temp->first);
					if: #name == null;
						local: 'name' = #temp->first;
					/if;
					if: (#Filter_Multi >> #name);
						#Link_Array->(Insert: #name = #temp->second);
					else: (#Filter_Action >> #name);
						#Link_Action = #name;
					else: #temp->second === null;
						// Nothing
					else: #temp->second->(isa: 'string') && (#temp->second == '');
						// Nothing
					else: !(#Filter_Global >> #name);
						#Link_Map->(Insert: #name = #temp->second);
					/if;	
				else;
					#Link_Array->(Insert: #temp);
				/if;
			else;
				if: #temp->(isa: 'string') && #temp->(beginswith: '-No');
					if: (#temp == '-NoSort');
						#Link_Array->(RemoveAll: '-SortOrder');
						#Link_Array->(RemoveAll: '-SortField');
					else: (#temp == '-NoAction');
						Local: 'Link_Action' = '';
					else: (#temp == '-NoClassic');
						Local: 'Link_Action' = '';
						iterate: #Filter_Classic, (local: 'temp2');
							#Link_Map->(Remove: #temp2);
						/iterate;
					else: (#temp == '-NoParams') || (#temp == '-NoSearch');
						Local: 'SortField_Index' = #Link_Array->(FindIndex: '-SortField');
						Local: 'SortOrder_Index' = #Link_Array->(FindIndex: '-SortOrder');
						Loop: -LoopFrom=#Link_Array->Size, -LoopTo=1, -LoopIncrement=(-1);
							If: (#SortField_Index !>> Loop_Count) && (#SortOrder_Index !>> Loop_Count);
								#Link_Array->(Remove: Loop_Count);
							/If;
						/Loop;						
					else: (#temp == '-NoTokens');
						Local: 'Temp_Map' = #Link_Map;
						Local: 'Link_Map' = (map);
						iterate: #Temp_Map, (local: 'temp');
							if: !(#temp->first >> '-token');
								#Link_Map->(insert: #temp->first = #temp->second);
							/if;
						/iterate;
					else: #temp->(beginswith: '-No.');
						Local: 'Name' = #temp - '-No.';
						Local: 'Name_Index' = #Link_Array->(FindIndex: #Name);
						Loop: -LoopFrom=#Name_Index->Size, -LoopTo=1, -LoopIncrement=(-1);
							Local: 'Index' = #Name_Index->(Get: Loop_Count);
							#Link_Array->(Remove: #Index);
							If: (#Index > 1) && ((#Link_Array->(Get: #Index - 1) == '-Op') || (#Link_Array->(Get: #Index - 1) == -Required));
								#Link_Array->(Remove: #Index - 1);
								If: (#Index > 2) && ((#Link_Array->(Get: #Index - 2) == '-Op') || (#Link_Array->(Get: #Index - 2) == -Required));
									#Link_Array->(Remove: #Index - 2);
								/If;
							/If;
						/Loop;
					else;
						local: 'name' = #Equivalence->(Find: #temp - 'No');
						if: #name == null;
							local: 'name' = #temp - '-No';
						/if;
						#Link_Map->(Remove: #name);
						#Link_Array->(RemoveAll: #name);
					/if;
				else: (#Filter_Action >> #temp);
					#Link_Action = #temp;
				/if;
			/if;
					
			#count += 1;
		/While;
		
		// Eliminate default values
		If: (#link_map->(find: '-OperatorLogical') == 'and');
			#link_map->(remove: '-OperatorLogical');
		/If;
		
		if: (#Link_Action == '') || (#Link_Action == '-Nothing');
			Local: 'param_array' = (array);
		Else;
			Local: 'param_array' = (array: #Link_Action = 'Action');
		/If;
		iterate: #link_map, (local: 'temp');
			#param_array->(insert: #temp);
		/iterate;
		#param_array->(merge: #link_array);
		
		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;
	
	Define_Tag: 'Link_URL', -Priority='Replace';
		
		if: !(local_defined: 'param_array');
			Local: 'param_array' = (Params);
			\Link_Params->(eval: -params=(array: -noreturn));
		/if;
		
		Local: 'Output' = '';
		
		If: #param_array >> '-Response';
			local: 'file' = (string: #param_array->(Find: '-Response')->(Get:1)->Second);
			#param_array->(RemoveAll: '-Response');
		else;
			local: 'file' = (response_filepath);
		/if;
		
		if: #file->(endswith: '.lasso') || #file->(endswith: '.lassoapp');
			#Output += #file + '?';
		else;
			#Output += 'Action.Lasso?-Response=' + (encode_stricturl: #file) + '&';
		/if;
	
		if: #param_array->size > 0;
			Iterate: #param_array, (Local: 'temp');
				#Temp = (Encode_StrictURL: #Temp->First) + '=' + (Encode_StrictURL: #Temp->Second);
			/Iterate;
			#Output += #param_array->(Join: '&');
		else: (params >> -noempty);
			#Output = '';
		/if;
		
		#Output->(RemoveTrailing: '&');
		
		if: (params !>> -noreturn);
			Return: #Output;
		/if;
	/Define_Tag;

	Define_Tag: 'Link', -Container, -Priority='Replace';

		\Link_URL->(eval: -params=(array: -noreturn));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;

	/Define_Tag;

	Define_Tag: 'Link_SetFormat', -priority='replace';
		Variable: '_Link_Format' = (Params);
		$_Link_Format->(insert: null);
	/Define_Tag;

	//
	// Params Tags
	//

	Define_Tag: 'Link_CurrentActionParams', -priority='replace';
		
		Local: 'action' = (lasso_currentaction);

		Local: 'param_array' = (action_params);
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		if: (local_defined: 'params') && (#params->(isa: 'array')) && (#params->size > 0);
			#param_array->(merge: #params);
		/if;
		\Link_Params->(eval: -params=(array: -noreturn));

		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;

	Define_Tag: 'Link_DetailParams', -priority='replace';
		
		Local: 'KeyField' = (string: KeyField_Name);
		Local: 'KeyValue' = (string: KeyField_Value);
		if: (#KeyField != '') && (#KeyValue == '');
			Local: 'KeyValue' = (string: (Field: #KeyField));
		/if;
		if: (#KeyValue == '') || (#KeyValue == (-1));
			Local: 'param_array' = (array);
		else;
			Local: 'param_array' = (action_params);
			if: (#keyfield != '');
				#param_array->(insert: -KeyField=#keyfield);
			/if;
			#param_array->(insert: -KeyValue=#keyvalue);
			#param_array->(merge: (Array: -NoSkipRecords, -NoMaxRecords, -NoParams, -NoSort, -NoAction));
			#param_array->(insert: -Search='Action');
			If: !(Local_Defined: 'Params');
				Local: 'Params' = (Params);
			/If;
			if: (local_defined: 'params') && (#params->(isa: 'array')) && (#params->size > 0);
				#param_array->(merge: #params);
			/if;
			\Link_Params->(eval: -params=(array: -noreturn));
		/if;
		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;

	Define_Tag: 'Link_CurrentRecordParams', -priority='replace';
		
		Local: 'action' = (lasso_currentaction);
		Local: 'loop_count' = (math_max: 1, (integer: $__record_count__->last));
		Local: 'shown_count' = (integer: shown_count);
		Local: 'shown_first' = (integer: shown_first);
		Local: 'skiprecords' = (#shown_first + #loop_count - 2);

		If: (#Shown_Count == 0) || ((#action != 'search') && (#action != 'findall'));
			Local: 'param_array' = (array);
		Else;
			Local: 'param_array' = (action_params);
			#param_array->(insert: -MaxRecords=1);
			#param_array->(insert: -SkipRecords=#skiprecords);
			If: !(Local_Defined: 'Params');
				Local: 'Params' = (Params);
			/If;
			if: (local_defined: 'params') && (#params->(isa: 'array')) && (#params->size > 0);
				#param_array->(merge: #params);
			/if;
			\Link_Params->(eval: -params=(array: -noreturn));
		/If;
		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;

	Define_Tag: 'Link_NextRecordParams', -priority='replace';
		
		Local: 'action' = (lasso_currentaction);
		Local: 'loop_count' = (math_max: 1, (integer: $__record_count__->last));
		Local: 'shown_first' = (integer: shown_first);
		Local: 'found_count' = (integer: found_count);
		Local: 'skiprecords' = (#shown_first + #loop_count - 1);

		If: (#found_count == 0) || (#skiprecords >= #found_count) || ((#action != 'search') && (#action != 'findall'));
			Local: 'param_array' = (array);
		Else;
			Local: 'param_array' = (action_params);
			#param_array->(insert: -MaxRecords=1);
			#param_array->(insert: -SkipRecords=#skiprecords);
			If: !(Local_Defined: 'Params');
				Local: 'Params' = (Params);
			/If;
			if: (local_defined: 'params') && (#params->(isa: 'array')) && (#params->size > 0);
				#param_array->(merge: #params);
			/if;
			\Link_Params->(eval: -params=(array: -noreturn));
		/If;
		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;

	Define_Tag: 'Link_PrevRecordParams', -priority='replace';
		
		Local: 'action' = (lasso_currentaction);
		Local: 'loop_count' = (math_max: 1, (integer: $__record_count__->last));
		Local: 'found_count' = (integer: found_count);
		Local: 'shown_first' = (integer: shown_first);
		Local: 'skiprecords' = (#shown_first + #loop_count - 3);

		If: (#found_count == 0) || (#skiprecords < 0) || ((#action != 'search') && (#action != 'findall'));
			Local: 'param_array' = (array);
		Else;
			Local: 'param_array' = (action_params);
			#param_array->(insert: -MaxRecords=1);
			#param_array->(insert: -SkipRecords=#skiprecords);
			If: !(Local_Defined: 'Params');
				Local: 'Params' = (Params);
			/If;
			if: (local_defined: 'params') && (#params->(isa: 'array')) && (#params->size > 0);
				#param_array->(merge: #params);
			/if;
			\Link_Params->(eval: -params=(array: -noreturn));
		/If;
		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;

	Define_Tag: 'Link_FirstRecordParams', -priority='replace';
		
		Local: 'action' = (lasso_currentaction);
		Local: 'found_count' = (integer: found_count);
		Local: 'skiprecords' = 0;

		If: (#found_count == 0) || ((#action != 'search') && (#action != 'findall'));
			Local: 'param_array' = (array);
		Else;
			Local: 'param_array' = (action_params);
			#param_array->(insert: -MaxRecords=1);
			#param_array->(insert: -SkipRecords=#skiprecords);
			If: !(Local_Defined: 'Params');
				Local: 'Params' = (Params);
			/If;
			if: (local_defined: 'params') && (#params->(isa: 'array')) && (#params->size > 0);
				#param_array->(merge: #params);
			/if;
			\Link_Params->(eval: -params=(array: -noreturn));
		/If;
		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;

	Define_Tag: 'Link_LastRecordParams', -priority='replace';
		
		Local: 'action' = (lasso_currentaction);
		Local: 'found_count' = (integer: found_count);
		Local: 'skiprecords' = (#found_count - 1);

		If: (#found_count == 0) || ((#action != 'search') && (#action != 'findall'));
			Local: 'param_array' = (array);
		Else;
			Local: 'param_array' = (action_params);
			#param_array->(insert: -MaxRecords=1);
			#param_array->(insert: -SkipRecords=#skiprecords);
			If: !(Local_Defined: 'Params');
				Local: 'Params' = (Params);
			/If;
			if: (local_defined: 'params') && (#params->(isa: 'array')) && (#params->size > 0);
				#param_array->(merge: #params);
			/if;
			\Link_Params->(eval: -params=(array: -noreturn));
		/If;
		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;

	Define_Tag: 'Link_CurrentSearchParams', -priority='replace';
		
		Local: 'action' = (lasso_currentaction);
		Local: 'shown_count' = (integer: shown_count);

		If: (#Shown_Count == 0) || ((#action != 'search') && (#action != 'findall'));
			Local: 'param_array' = (array);
		Else;
			Local: 'param_array' = (action_params);
			If: !(Local_Defined: 'Params');
				Local: 'Params' = (Params);
			/If;
			if: (local_defined: 'params') && (#params->(isa: 'array')) && (#params->size > 0);
				#param_array->(merge: #params);
			/if;
			\Link_Params->(eval: -params=(array: -noreturn));
		/If;
		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;

	Define_Tag: 'Link_CurrentGroupParams', -priority='replace';
		
		Local: 'action' = (lasso_currentaction);
		Local: 'shown_count' = (integer: shown_count);

		If: (#Shown_Count == 0) || ((#action != 'search') && (#action != 'findall'));
			Local: 'param_array' = (array);
		Else;
			Local: 'param_array' = (action_params);
			If: !(Local_Defined: 'Params');
				Local: 'Params' = (Params);
			/If;
			if: (local_defined: 'params') && (#params->(isa: 'array')) && (#params->size > 0);
				#param_array->(merge: #params);
			/if;
			\Link_Params->(eval: -params=(array: -noreturn));
		/If;
		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;

	Define_Tag: 'Link_NextGroupParams', -priority='replace';
		
		Local: 'action' = (lasso_currentaction);
		Local: 'found_count' = (integer: found_count);
		Local: 'skiprecords' = (integer: skiprecords_value);
		Local: 'maxrecords' = (integer: maxrecords_value);
		Local: 'skiprecords' = (#skiprecords + #maxrecords);

		If: (#found_count == 0) || (#skiprecords >= #found_count) || ((#action != 'search') && (#action != 'findall'));
			Local: 'param_array' = (array);
		Else;
			Local: 'param_array' = (action_params);
			#param_array->(insert: -SkipRecords=#skiprecords);
			If: !(Local_Defined: 'Params');
				Local: 'Params' = (Params);
			/If;
			if: (local_defined: 'params') && (#params->(isa: 'array')) && (#params->size > 0);
				#param_array->(merge: #params);
			/if;
			\Link_Params->(eval: -params=(array: -noreturn));
		/If;
		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;

	Define_Tag: 'Link_PrevGroupParams', -priority='replace';
		
		Local: 'action' = (lasso_currentaction);
		Local: 'found_count' = (integer: found_count);
		Local: 'skiprecords' = (integer: skiprecords_value);
		Local: 'maxrecords' = (integer: maxrecords_value);
		Local: 'skiprecords' = (#skiprecords - #maxrecords);

		If: (#found_count == 0) || (#skiprecords < 0);
			Local: 'param_array' = (array);
		Else;
			Local: 'param_array' = (action_params);
			#param_array->(insert: -SkipRecords=#skiprecords);
			If: !(Local_Defined: 'Params');
				Local: 'Params' = (Params);
			/If;
			if: (local_defined: 'params') && (#params->(isa: 'array')) && (#params->size > 0);
				#param_array->(merge: #params);
			/if;
			\Link_Params->(eval: -params=(array: -noreturn));
		/If;
		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;

	Define_Tag: 'Link_FirstGroupParams', -priority='replace';
		
		Local: 'action' = (lasso_currentaction);
		Local: 'found_count' = (integer: found_count);
		Local: 'skiprecords' = 0;

		If: (#found_count == 0) || ((#action != 'search') && (#action != 'findall'));
			Local: 'param_array' = (array);
		Else;
			Local: 'param_array' = (action_params);
			#param_array->(insert: -SkipRecords=#skiprecords);
			If: !(Local_Defined: 'Params');
				Local: 'Params' = (Params);
			/If;
			if: (local_defined: 'params') && (#params->(isa: 'array')) && (#params->size > 0);
				#param_array->(merge: #params);
			/if;
			\Link_Params->(eval: -params=(array: -noreturn));
		/If;
		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;

	Define_Tag: 'Link_LastGroupParams', -priority='replace';
		
		Local: 'action' = (lasso_currentaction);
		Local: 'found_count' = (integer: found_count);
		Local: 'maxrecords' = (integer: maxrecords_value);
		Local: 'skiprecords' = (#found_count - #maxrecords);
		if: (#skiprecords < 0);
			Local: 'skiprecords' = 0;
		/if;
		
		If: (#found_count == 0);
			Local: 'param_array' = (array);
		Else;
			Local: 'param_array' = (action_params);
			#param_array->(insert: -SkipRecords=#skiprecords);
			If: !(Local_Defined: 'Params');
				Local: 'Params' = (Params);
			/If;
			if: (local_defined: 'params') && (#params->(isa: 'array')) && (#params->size > 0);
				#param_array->(merge: #params);
			/if;
			\Link_Params->(eval: -params=(array: -noreturn));
		/If;
		if: (params !>> -noreturn);
			Return: #param_array;
		/if;
		
	/Define_Tag;

	//
	// URL Tags
	//
	
	Define_Tag: 'Link_DetailURL', -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		If: (#Params >> '-LinkSearchAgain');
			\Link_CurrentRecordParams->(eval: -params=(array: -noreturn));
		Else;
			\Link_DetailParams->(eval: -params=(array: -noreturn));
		/If;
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		return: #output;
	/Define_Tag;

	Define_Tag: 'Link_CurrentRecordURL', -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_CurrentRecordParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		return: #output;
	/Define_Tag;

	Define_Tag: 'Link_PrevRecordURL', -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_PrevRecordParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		return: #output;
	/Define_Tag;

	Define_Tag: 'Link_NextRecordURL', -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_NextRecordParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		return: #output;
	/Define_Tag;

	Define_Tag: 'Link_FirstRecordURL', -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_FirstRecordParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		return: #output;
	/Define_Tag;

	Define_Tag: 'Link_LastRecordURL', -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_LastRecordParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		return: #output;
	/Define_Tag;

	Define_Tag: 'Link_CurrentGroupURL', -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_CurrentGroupParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		return: #output;
	/Define_Tag;

	Define_Tag: 'Link_CurrentActionURL', -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_CurrentActionParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn));
		return: #output;
	/Define_Tag;
	
	Define_Tag: 'Link_CurrentSearchURL', -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_CurrentSearchParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn));
		return: #output;
	/Define_Tag;

	Define_Tag: 'Link_PrevGroupURL', -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_PrevGroupParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		return: #output;
	/Define_Tag;

	Define_Tag: 'Link_NextGroupURL', -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_NextGroupParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		return: #output;
	/Define_Tag;

	Define_Tag: 'Link_FirstGroupURL', -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_FirstGroupParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		return: #output;
	/Define_Tag;

	Define_Tag: 'Link_LastGroupURL', -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_LastGroupParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		return: #output;
	/Define_Tag;

	//
	// Container Tags
	//

	Define_Tag: 'Link_Detail', -container, -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		If: (#Params >> '-LinkSearchAgain');
			\Link_CurrentRecordParams->(eval: -params=(array: -noreturn));
		Else;
			\Link_DetailParams->(eval: -params=(array: -noreturn));
		/If;
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;
	/Define_Tag;

	Define_Tag: 'Link_CurrentRecord', -container, -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_CurrentRecordParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;
	/Define_Tag;

	Define_Tag: 'Link_PrevRecord', -container, -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_PrevRecordParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;
	/Define_Tag;

	Define_Tag: 'Link_NextRecord', -container, -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_NextRecordParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;
	/Define_Tag;

	Define_Tag: 'Link_FirstRecord', -container, -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_FirstRecordParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;
	/Define_Tag;

	Define_Tag: 'Link_LastRecord', -container, -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_LastRecordParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;
	/Define_Tag;

	Define_Tag: 'Link_CurrentGroup', -container, -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_CurrentGroupParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;
	/Define_Tag;

	Define_Tag: 'Link_CurrentAction', -container, -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_CurrentActionParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;
	/Define_Tag;

	Define_Tag: 'Link_CurrentSearch', -container, -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_CurrentSearchParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;
	/Define_Tag;

	Define_Tag: 'Link_PrevGroup', -container, -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_PrevGroupParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;
	/Define_Tag;

	Define_Tag: 'Link_NextGroup', -container, -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_NextGroupParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;
	/Define_Tag;

	Define_Tag: 'Link_FirstGroup', -container, -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_FirstGroupParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;
	/Define_Tag;

	Define_Tag: 'Link_LastGroup', -container, -priority='replace';
		If: !(Local_Defined: 'Params');
			Local: 'Params' = (Params);
		/If;
		\Link_LastGroupParams->(eval: -params=(array: -noreturn));
		\Link_URL->(eval: -params=(array: -noreturn, -noempty));
		if: (#output != '');
			return: '<a href="' + (encode_html: #output) + '">' + (Run_Children) + '</a>';
		/if;
	/Define_Tag;

?>