<?LassoScript

	// Security Tags
	//
	// These tage provide additional security options beyond those provided
	// by [Auth] and [Auth_Admin].
	//
	// Check that the current user belongs to a group or one of a set of groups
	// To check that a user belongs to multiple groups use multiple [Auth_Group] tags.
	//
	// [Auth_Group: 'group']
	// [Auth_Group: (array: 'group', 'group')]
	//
	// Check for a specific Lasso Security username or one of a set of usernames
	//
	// [Auth_User: 'username']
	// [Auth_User: (array: 'username', 'username')]
	//
	// Check against a custom username/password or an array of usernames/passwords.
	// If two arrays are specified then common elements in each array match as username/password
	// pairs.  Both arrays should have the same length.
	//
	// [Auth_Custom: 'username', 'password']
	// [Auth_Custom: (array: 'username', 'username'), 'password']
	// [Auth_Custom: (array: 'username', 'username'), (array: 'password', 'password')]
	// [Auth_Custom: 'username', (array: 'password', 'password')]
	//
	// All tags take an optional realm parameter (default "Lasso Security")
	// All tags take an optional -NoAbort parameter that deactivates the abort.
	//
	// [Auth_Custom: 'username', 'password', 'realm', -NoAbort]
	//
		
	Define_Tag: 'Auth_Group', -Priority='Replace',
			-Optional='Group',
			-Optional='Realm',
			-Optional='NoAbort';
		
		If: !(Local_Defined: 'Group');
			If: ((Admin_CurrentGroups) != (Array: 'ANYUSER'));
				Return;
			/If;	
		Else: (#Group->(IsA: 'array'));
			Iterate: #Group, (Local: 'Temp');
				If: ((Admin_CurrentGroups) >> (String: #Temp));
					Return;
				/If;
			/Iterate;
		Else: ((Admin_CurrentGroups) >> (String: #Group));
			Return;
		/If;
		
		If: !(Local_Defined: 'Realm');
			Local: 'Realm' = 'Lasso Security';
		/If;
		
		$__http_header__ =  'HTTP/1.0 401 Unauthorized\r\n';
		$__http_header__ += 'WWW-Authenticate: Basic realm="' + #Realm + '"\r\n';
		$__http_header__ += 'Server: Lasso Professional (' + (Lasso_Version) + ')\r\n';
		If: (Params !>> '-NoAbort');
			Abort;
		/If;
		
	/Define_Tag;

	Define_Tag: 'Auth_User', -Priority='Replace',
			-Optional='User',
			-Optional='Realm',
			-Optional='NoAbort';
		
		If: !(Local_Defined: 'User');
			If: ((Admin_CurrentUsername) != 'ANONYMOUS');
				Return;
			/If;	
		Else: (#User->(IsA: 'array')) && (#User >> (Admin_CurrentUsername));
			Return;
		Else: ((Admin_CurrentUsername) == (String: #User));
			Return;
		/If;
		
		If: !(Local_Defined: 'Realm');
			Local: 'Realm' = 'Lasso Security';
		/If;
		
		$__http_header__ =  'HTTP/1.0 401 Unauthorized\r\n';
		$__http_header__ += 'WWW-Authenticate: Basic realm="' + #Realm + '"\r\n';
		$__http_header__ += 'Server: Lasso Professional (' + (Lasso_Version) + ')\r\n';
		If: (Params !>> '-NoAbort');
			Abort;
		/If;
			
	/Define_Tag;

	Define_Tag: 'Auth_Custom', -Priority='Replace',
			-Required='User', 
			-Required='Password', 
			-Optional='Realm',
			-Optional='NoAbort';
		
		If: (#User->(IsA: 'array')) && (#User >> Client_Username);
			If: (#Password->(IsA: 'array'));
				Local: 'Indices' = #User->(FindIndex: Client_Username);
				Iterate: #Indices, (Local: 'Temp');
					If: (#Password->Size >= #Temp) && ((String: Client_Password)->(Equals: (String: #Password->(Get: #Temp)), -Case));
						Return;
					/If;				
				/Iterate;
			Else: ((String: Client_Password)->(Equals: (String: #Password), -Case));
				Return;
			/If;
		Else: (#Password->(IsA: 'array'));
			Iterate: #Password, (Local: 'Temp');
				If: ((String: Client_Password)->(Equals: (String: #Password), -Case));
					Return;
				/If;				
			/Iterate;
		Else: (String: Client_Username)->(Equals: (String: #User)) && (String: Client_Password)->(Equals: (String: #Password), -Case);
			Return;
		/If;
		
		If: !(Local_Defined: 'Realm');
			Local: 'Realm' = 'Lasso Security';
		/If;
		
		$__http_header__ =  'HTTP/1.0 401 Unauthorized\r\n';
		$__http_header__ += 'WWW-Authenticate: Basic realm="' + #Realm + '"\r\n';
		$__http_header__ += 'Server: Lasso Professional (' + (Lasso_Version) + ')\r\n';
		If: (Params !>> '-NoAbort');
			Abort;
		/If;
	
	/Define_Tag;


	// Admin Tags
	//
	// These tage allow manipulations of users and groups by individual users and
	// group administrators.  They use -Privileged permission to modify the internal
	// security settings.
	//
	// [Admin_ChangeUser: 'Username', 'Old Password', 'New Password'] -> True/False
	// [Admin_CreateUser: 'Usenrame', 'Password'] -> True/False
	// [Admin_GroupAssignUser: 'Group', 'Username'] -> True/False
	// [Admin_GroupListUsers: 'Group'] -> True/False
	// [Admin_GroupRemoveUser: 'Group', 'Username']
	// [Admin_ListGroups]

	Define_Tag: 'Admin_ChangeUser', -Privileged, -Priority='Replace',
			-Required='Username',
			-Required='OldPassword',
			-Required='NewPassword';
	
		local: 'output' = false;

		inline: -search,
				-database='lasso_internal',
				-table='security_users',
				-keyfield='id',
				-op='eq', 'name' = #username,
				-op='eq', 'password' = (encrypt_md5: #oldpassword),
				-op='eq', 'enabled'='Y',
				-maxrecords='all';
			if: (found_count > 0);
				inline: -update,
						-database=(database_name),
						-table=(table_name),
						-keyfield=(keyfield_name),
						-keyvalue=(keyfield_value),
						'password' = (encrypt_md5: #newpassword);
					if: error_code == 0;
						local: 'output' = true;
						_security_flushcaches;
					/if;
				/inline;
			/if;
		/inline;
		
		return: #output;			
	
	/Define_Tag;

	Define_Tag: 'Admin_CreateUser', -Privileged, -Priority='Replace',
			-Required='Username',
			-Required='Password';
	
		local: 'output' = false;
		local: 'groups' = (admin_listgroups);
		if: (#groups === false) || (#groups->size == 0);
			return: 'false';
		/if;

		inline: -search,
				-database='lasso_internal',
				-table='security_users',
				-keyfield='id',
				-op='eq', 'name' = #username,
				-op='eq', 'enabled'='Y',
				-maxrecords='all';
			if: (found_count == 0);
				inline: -add,
						-database=(database_name),
						-table=(table_name),
						-keyfield=(keyfield_name),
						'name' = #username,
						'password' = (encrypt_md5: #password),
						'id_site' = 1,
						'enabled'='Y';
					if: error_code == 0;
						local: 'output' = true;
						_security_flushcaches;
					/if;
				/inline;
			/if;
		/inline;
	
		return: #output;			
	
	/Define_Tag;

	Define_Tag: 'Admin_GroupAssignUser', -Privileged, -Priority='Replace',
			-Required='Group',
			-Required='Username';
	
		local: 'output' = true;
		if: !((admin_listgroups) >> #group);
			return: false;
		/if;

		inline: -search,
				-database='lasso_internal',
				-table='security_users',
				-keyfield='id',
				-op='eq', 'name' = #username,
				-op='eq', 'enabled'='Y',
				-maxrecords='all';
			if: (found_count > 0);
				local: 'user_id' = (field: 'id');
			else;
				local: 'output' = false;
			/if;
		/inline;

		if: (#output === false);
			return: false;
		/if;

		inline: -search,
				-database='lasso_internal',
				-table='security_groups',
				-keyfield='id',
				-op='eq', 'name' = #group,
				-op='eq', 'enabled'='Y',
				-maxrecords='all';
			if: (found_count > 0);
				local: 'group_id' = (field: 'id');
			else;
				local: 'output' = false;
			/if;
		/inline;
		
		if: (#output === false);
			return: false;
		/if;

		local: 'output' = false;
		
		inline: -search,
				-database='lasso_internal',
				-table='security_ug_map',
				-keyfield='id',
				-op='eq', 'id_user' = #user_id,
				-op='eq', 'id_group' = #group_id,
				-maxrecords='all';
			if: (found_count > 0);
				local: 'output' = true;
			/if;
		/inline;
			
		if: (#output === true);
			return: true;
		/if;

		inline: -add,
				-database='lasso_internal',
				-table='security_ug_map',
				-keyfield='id',
				'id_user' = #user_id,
				'id_group' = #group_id;
			if: error_code == 0;
				local: 'output' = true;
				_security_flushcaches;
			/if;
		/inline;
		
		return: #output;	
				
	/Define_Tag;

	Define_Tag: 'Admin_GroupListUsers', -Privileged, -Priority='Replace',
			-Required='Group';
	
		if: !((admin_listgroups) >> #group);
			return: false;
		/if;

		local: 'output' = (array);
		
		inline: -search,
				-database='lasso_internal',
				-table='security_groups',
				-keyfield='id',
				-op='eq', 'name' = #group,
				-op='eq', 'enabled'='Y',
				-maxrecords='all';
			if: (found_count > 0);
				local: 'group_id' = (field: 'id');
			else;
				local: 'output' = false;
			/if;
		/inline;

		if: (#output === false);
			return: false;
		/if;

		inline: -search,
				-database='lasso_internal',
				-table='security_ug_map',
				-keyfield='id',
				-op='eq', 'id_group' = #group_id,
				-maxrecords='all';
			records;
				inline: -search,
						-database='lasso_internal',
						-table='security_users',
						-keyfield='id',
						-op='eq', 'id' = (field: 'id_user'),
						-op='eq', 'enabled'='Y',
						-maxrecords='all';
					if: (found_count > 0) && ((field: 'name') != '');
						#output->(insert: (field: 'name'));	
					/if;
				/inline;
			/records;
		/inline;

		return: #output;
		
	/Define_Tag;
	
	Define_Tag: 'Admin_UserListGroups', -Privileged, -Priority='Replace',
			-Required='Username';
	
		local: 'groups' = admin_listgroups;
		local: 'output' = (array);
		
		inline: -search,
				-database='lasso_internal',
				-table='security_users',
				-keyfield='id',
				-op='eq', 'name' = #username,
				-op='eq', 'enabled'='Y',
				-maxrecords='all';
			if: (found_count > 0);
				local: 'user_id' = (field: 'id');
			else;
				local: 'output' = false;
			/if;
		/inline;

		if: (#output === false);
			return: false;
		/if;

		inline: -search,
				-database='lasso_internal',
				-table='security_ug_map',
				-keyfield='id',
				-op='eq', 'id_user' = #user_id,
				-maxrecords='all';
			records;
				inline: -search,
						-database='lasso_internal',
						-table='security_groups',
						-keyfield='id',
						-op='eq', 'id' = (field: 'id_group'),
						-op='eq', 'enabled'='Y',
						-maxrecords='all';
					if: (found_count > 0) && ((field: 'name') != '') && (#groups >> (field: 'name'));
						#output->(insert: (field: 'name'));	
					/if;
				/inline;
			/records;
		/inline;

		return: #output;
		
	/Define_Tag;

	Define_Tag: 'Admin_GroupRemoveUser', -Privileged, -Priority='Replace',
			-Required='Group',
			-Required='Username';
	
		local: 'output' = null;
		
		if: !((admin_listgroups) >> #group);
			return: false;
		/if;
	
		inline: -search,
				-database='lasso_internal',
				-table='security_users',
				-keyfield='id',
				-op='eq', 'name' = #username,
				-op='eq', 'enabled'='Y',
				-maxrecords='all';
			if: (found_count > 0);
				local: 'user_id' = (field: 'id');
			else;
				local: 'output' = false;
			/if;
		/inline;

		if: (#output === false);
			return: false;
		/if;

		inline: -search,
				-database='lasso_internal',
				-table='security_groups',
				-keyfield='id',
				-op='eq', 'name' = #group,
				-op='eq', 'enabled'='Y',
				-maxrecords='all';
			if: (found_count > 0);
				local: 'group_id' = (field: 'id');
			else;
				local: 'output' = false;
			/if;
		/inline;
		
		if: (#output === false);
			return: false;
		/if;

		inline: -search,
				-database='lasso_internal',
				-table='security_ug_map',
				-keyfield='id',
				-op='eq', 'id_user' = #user_id,
				-op='eq', 'id_group' = #group_id,
				-maxrecords='all';
			records;
				inline: -delete,
						-database=(database_name),
						-table=(table_name),
						-keyfield=(keyfield_name),
						-keyvalue=(keyfield_value);
				/inline;
			/records;
		/inline;
		
		_security_flushcaches;
		return: true;	
	
	/Define_Tag;

	Define_Tag: 'Admin_ListGroups', -Privileged, -Priority='Replace';
	
		local: 'output' = (array);
		
		inline: -search,
				-database='lasso_internal',
				-table='security_users',
				-keyfield='id',
				-op='eq', 'name' = (admin_currentusername),
				-op='eq', 'enabled'='Y',
				-maxrecords='all';
			if: (found_count > 0);
				local: 'user_id' = (field: 'id');
			else;
				local: 'output' = false;
			/if;
		/inline;

		if: (#output === false);
			return: #output;
		/if;

		if: #user_id == 2;
			inline: -search,
					-database='lasso_internal',
					-table='security_groups',
					-keyfield='id',
					-op='eq', 'enabled'='Y',
					-maxrecords='all';
				records;
					if: ((field: 'name') != '');
						#output->(insert: (field: 'name'));	
					/if;
				/records;
			/inline;
		else;
			inline: -search,
					-database='lasso_internal',
					-table='security_ug_map',
					-keyfield='id',
					-op='eq', 'id_user' = #user_id,
					-op='eq', 'admin' = 'Y',
					-maxrecords='all';
				records;
					inline: -search,
							-database='lasso_internal',
							-table='security_groups',
							-keyfield='id',
							-op='eq', 'id' = (field: 'id_group'),
							-op='eq', 'enabled'='Y',
							-maxrecords='all';
						if: ((field: 'name') != '');
							#output->(insert: (field: 'name'));	
						/if;
					/inline;
				/records;
			/inline;
		/if;
		
		return: #output;
	
	/Define_Tag;
?>